-- "Habitat Wars"
-- By Tim Sandford, 2021
-- Comments to timsandford42@hotmail.com

require("file42")
require("math42")
require("music42")
require("table42")

function love.run()   -- Main function
  gVERSION, gPATCH, gREVISION, gCODENAME = love.getVersion()
  if gVERSION >=  11 then  -- Colour fix for 11.0 onwards
    local oldSetColor = love.graphics.setColor
    love.graphics.setColor = function (r, g, b, a)     
      if type(r)=="table" then
        r, g, b, a = r[1] / 255, r[2] / 255, r[3] / 255, (r[4] or 255) / 255
      else
        r, g, b, a = r / 255, g / 255, b / 255, (a or 255) / 255
      end
      oldSetColor(r,g,b,a)
    end
  end
  math.randomseed(os.time())  -- True randomness
  love.load(love.arg.parseGameArguments(arg), arg) -- load variables etc.
  while true do  -- Main loop
    if love.event then -- Process events
      love.event.pump()
      for name, a,b,c,d,e,f in love.event.poll() do
        if name == "quit" then
          if not love.quit or not love.quit() then
            return a
          end
        end
        love.handlers[name](a,b,c,d,e,f)
      end
    end
    love.update() -- Update
    love.graphics.origin() -- Not sure, probably reset origin?
    love.graphics.clear(love.graphics.getBackgroundColor()) -- Clears the screen
    love.draw() -- Collect draw commands
    love.graphics.present() -- Present the graphics on the screen!
    love.timer.sleep(0.001) -- This is to lower the load on the CPU, but I could edit it :)	
  end
end

function love.load(arg)  -- Load everything in
  local i, j, found
  io.stdout:setvbuf("no")                                         -- Print messages immediately, allows tracing bugs
  if arg[#arg] == "-debug" then require("mobdebug").start() end   -- Zerobrane debugging
  gSCREEN_W, gSCREEN_H = love.graphics.getWidth(), love.graphics.getHeight() -- Get screensize
  gRATIOX, gRATIOY = gSCREEN_W/800, gSCREEN_H/600                  -- Scale graphics to screen
  gFADE, gFADESPEED = 255, 5                                       -- Fade in effect
  gTIMER = love.timer.getTime()                                    -- Use timer in love.update for speed of game
  gFLASH, gFLASH_z, gFLASH_min, gFLASH_max = 255, 10, 80, 255		  -- Flashing images/text transparency
  gZOOMED, gDRAGGING, gMOUSEGRAB = false, false, 0                -- Used for dragging and selecting cards

  love.mouse.setCursor(love.mouse.newCursor("gui/cursor.png", 0, 0))

  fTINY = love.graphics.newFont("fonts/font.ttf", 10)             -- tiny font
  fSMALL = love.graphics.newFont("fonts/font.ttf", 14)            -- small font
  fDEFAULT = love.graphics.newFont("fonts/font.ttf", 19)          -- default font
  fHEADING = love.graphics.newFont("fonts/font.ttf", 28)          -- heading font
  fTITLE = love.graphics.newFont("fonts/font.ttf", 40)            -- title font
  fHUGE = love.graphics.newFont("fonts/font.ttf", 80)             -- huge font

  iICON, iBACKGROUND, iMAP, sSOUND, iIMAGE = {}, {}, {}, {}, {}
  for i = 1, 199, 1 do
    if love.filesystem.getInfo("gui/icon"..i..".png", "file") then iICON[i] = love.graphics.newImage("gui/icon"..i..".png") end
    if love.filesystem.getInfo("map/map"..i..".jpg", "file") then iMAP[i] = love.graphics.newImage("map/map"..i..".jpg") end
    if love.filesystem.getInfo("sounds/sound"..i..".ogg", "file") then sSOUND[i] = love.audio.newSource("sounds/sound"..i..".ogg", "static") end
    if love.filesystem.getInfo("images/pic"..i..".jpg", "file") then iIMAGE[i] = love.graphics.newImage("images/pic"..i..".jpg") end
    if love.filesystem.getInfo("backgrounds/background"..i..".jpg", "file") then iBACKGROUND[i] = love.graphics.newImage("backgrounds/background"..i..".jpg") end
  end
  iCARD = love.graphics.newImage("gui/card.png")
  iSQUARE = love.graphics.newImage("gui/square.png")
  iUNKNOWN = love.graphics.newImage("gui/unknown.jpg")

  if love.filesystem.getInfo("settings.txt", "file") then tSETTINGS = readsingletable("settings.txt")
  else tSETTINGS = readsingletable("data/defaultsettings.txt") end

  if love.filesystem.getInfo("player.txt", "file") then tPLAYER = readsingletable("player.txt")
  else tPLAYER = readsingletable("data/newplayer.txt") end

  tMAP = readmultitable("data/map.txt")
  tBLUEPRINT = readmultitable("data/cards.txt")
  tABILITIES = readmultitable("data/abilities.txt")
  tEFFECTS = readmultitable("data/effects.txt")
  tPATTERN = readmultitable("data/moves.txt")

  if love.filesystem.getInfo("collection.txt", "file") then tCOLLECTION = loaddeck("collection.txt")
  else tCOLLECTION = loaddeck("data/starterdeck.txt") end
  if love.filesystem.getInfo("unlocked.txt", "file") then tUNLOCKED = readsinglelist("unlocked.txt")
  else tUNLOCKED = readsinglelist("data/starterdeck.txt") end
  savedeck(tCOLLECTION, "collection.txt")
  writesinglelist("unlocked.txt", tUNLOCKED)

  musicload(tSETTINGS.MusicVol)
  startmenu()
end

function startmenu()  -- Load the main menu
  gGAMEMODE = "menu"
  love.mouse.setVisible(true)
  if tSETTINGS.Music=="Yes" then musicplay(1) end
  gFADE = 255
end

function loaddeck(filename)  -- Load a deck, returns as a list
  local list, i, j
  deck = {}
  if love.filesystem.getInfo(filename) then
    list = readsinglelist(filename)
    if next(list)~=nil and next(tBLUEPRINT)~=nil then
      for i = 1, #list, 1 do
        for j = 1, #tBLUEPRINT, 1 do
          if tBLUEPRINT[j].Name~=nil and tBLUEPRINT[j].Name==list[i] then table.insert(deck, deepcopy(tBLUEPRINT[j])) end
        end
      end
    end
  end
  deck = getlistofcards(deck)
  return deck
end

function savedeck(deck, filename)   -- Saves the names as a single list
  local list = {}
  local i
  if next(deck)~=nil then
    deck = getlistofcards(deck, "All", 0, "Name")
    for i = 1, #deck, 1 do
      list[i] = deck[i].Name
    end
  end
  writesinglelist(filename, list)
end

function newgame()  -- Start a new game
  local i, j, k
  gTURN, gDAYNIGHT, gWAIT = math.random(2), math.random(10), 0
  gNEWLEVEL, gNEWCARD, gNEWGOLD, gROUND = 0, 0, 0, 0
  xBOARD, yBOARD, gDISPLAYHAND = 250, 300, true
  tCLICK, gZOOMED, gDRAGGING = {}, false, false
  if gLEVEL~=0 then gBACKGROUND = gLEVEL else gBACKGROUND = math.random(11) end
  blankzones()
  blankgrid()
  tGAME.Score = {0, 0}
  tGAME.Winner, tGAME.Loser = 0, 0
  tCOMBAT, tGRAVE = {}, {}
  for j = 1, 2, 1 do
    tCOMBAT[j], tGRAVE[j] = {}, {}
    arrangehand(j)
    if next(tDECK[j])~=nil then
      for i = 1, #tDECK[j], 1 do
        tDECK[j][i].Available = false
        tDECK[j][i].GX, tDECK[j][i].GY = 0, 0
        tDECK[j][i].Wounds = 0
        tDECK[j][i].Animate = false
        tDECK[j][i].AniCount = 0
        tDECK[j][i].OriginalX, tDECK[j][i].OriginalY = tDECK[j][i].X, tDECK[j][i].Y
        tDECK[j][i].TargetX, tDECK[j][i].TargetY = tDECK[j][i].X, tDECK[j][i].Y
        tDECK[j][i].FloatyText = {}
        tDECK[j][i].Effect = {}
        for k = 1, #tEFFECTS, 1 do
          tDECK[j][i].Effect[k] = 0
        end
      end
    end
  end
  gGAMEMODE = "game"
  if tSETTINGS.Music=="Yes" then musicplay(2) end
  newturn()
end

function arrangehand(pl)  -- Layout cards in hand
  local yHAND = {460, 160}
  local rows, cols = getdimensions(tGAME.Pieces)
  local icon_size = math.min(260/rows, 260/cols)
  local x, y = 1, 1
  local i
  if next(tDECK[pl])~=nil then
    for i = 1, #tDECK[pl], 1 do
      tDECK[pl][i].X = 650-(((cols+1)/2)*icon_size)+(x*icon_size)
      tDECK[pl][i].Y = yHAND[pl]-(((rows+1)/2)*icon_size)+(y*icon_size)
      x = x + 1
      if x>cols then
        x = 1
        y = y + 1
      end
    end
  end
end

function arrangecombat()    -- Arrange the combat pieces (keep with the moving board)
  local i, j
  for j = 1, 2, 1 do
    if next(tCOMBAT[j])~=nil then
      for i = 1, #tCOMBAT[j], 1 do
        tCOMBAT[j][i].X, tCOMBAT[j][i].Y = gridtoscreen(tCOMBAT[j][i].GX, tCOMBAT[j][i].GY)
      end
    end
  end
end

function blankgrid()  -- Blank the grid
  local i, j
  tGRID = {}
  for j = 1, tGAME.Gridsize, 1 do
    tGRID[j] = {}
    for i = 1, tGAME.Gridsize, 1 do
      tGRID[j][i] = false
    end
  end
end

function blankzones()  -- Clear the scoring zones
  local i
  tZONE = {}
  for i = 1, tGAME.Gridsize, 1 do
    tZONE[i] = 0
  end
end  

function calculatezones()   -- Check for changes in scoring zones
  local j = (tGAME.Gridsize+1)/2
  local i, pl, index
  for i = 1, tGAME.Gridsize, 1 do
    pl, index = getsquare(i, j)
    if pl~=0 then
      if gethasability(tCOMBAT[pl][index], "Territorial") and tZONE[i]~=pl and math.random(2)==1 then
        seteffect("Immobile", 0, "Combat", pl, index, "Min")
        addfloatytext(tCOMBAT[pl][index], "+Territorial+")
      end
      tZONE[i]=pl
    end
  end
end

function scorezones()  -- Update the score
  for i = 1, tGAME.Gridsize, 1 do
    if tZONE[i]~=0 then tGAME.Score[tZONE[i]] = tGAME.Score[tZONE[i]] + 1 end
  end
end

function newturn()  -- Start a new turn
  local i, j, a, b, pl, index, e, dead
  gTURN = 3 - gTURN
  scorezones()
  a, b = gTURN, 3 - gTURN    -- friendly & foe
  gROUND = gROUND + 1
  gDAYNIGHT = gDAYNIGHT + 1
  if gDAYNIGHT>10 then gDAYNIGHT = 1 end
  if tCONTROL[a]=="Player" then
    tCLICK, gZOOMED, gDRAGGING, gACTION = {}, false, false, "" 
    highlightgrid()
  end
  AIsimulation=false
  if next(tDECK[a])~=nil then
    for i = #tDECK[a], 1, -1 do
      for j = 1, #tEFFECTS, 1 do
        if tDECK[a][i].Effect[j]<100 then tDECK[a][i].Effect[j]=math.max(tDECK[a][i].Effect[j]-1,0) end
      end
      if math.ceil(gROUND/2)>=tDECK[a][i].Tier then tDECK[a][i].Available=true
      else tDECK[a][i].Available=false end
    end
  end
  if next(tCOMBAT[a])~=nil then
    for i = #tCOMBAT[a], 1, -1 do
      dead = false
      if gethaseffect(tCOMBAT[a][i], "Venom") then
        addfloatytext(tCOMBAT[a][i], "-Venom-")
        dead = damage(tCOMBAT[a][i], 1, true)
      end
      if dead == false then 
        for j = 1, #tEFFECTS, 1 do
          if tCOMBAT[a][i].Effect[j]<100 then tCOMBAT[a][i].Effect[j]=math.max(tCOMBAT[a][i].Effect[j]-1,0) end
        end
        if gethasability(tCOMBAT[a][i], "Nocturnal") then
          if gDAYNIGHT>=6 then 
            seteffect("Nighttime", 1, "Combat", a, i, "Max")
            addfloatytext(tCOMBAT[a][i], "+Nocturnal+")          
          else 
            seteffect("Daytime", 1, "Combat", a, i, "Max") 
            addfloatytext(tCOMBAT[a][i], "-Nocturnal-")          
          end
        end
        if gethasability(tCOMBAT[a][i], "Lazy") and math.random(100)<=25 then
          seteffect("Immobile", 1, "Combat", a, i, "Max")
          addfloatytext(tCOMBAT[a][i], "-Lazy-")
        end
        if gethasability(tCOMBAT[a][i], "Slow") and math.random(100)<=33 then
          seteffect("Immobile", 1, "Combat", a, i, "Max")
          addfloatytext(tCOMBAT[a][i], "-Slow-")
        end
        if gethasability(tCOMBAT[a][i], "Night Vision") and gDAYNIGHT>=6 then
          if next(tCOMBAT[b])~=nil then
            e = geteffectbyname("Hidden")
            for j = 1, #tCOMBAT[b], 1 do
              if tCOMBAT[b][j].Effect[e]>0 then
                addfloatytext(tCOMBAT[a][i], "+Night Vision+")
                addfloatytext(tCOMBAT[b][j], "-Revealed-")
                tCOMBAT[b][j].Effect[e]=0
              end
            end
          end
        elseif gethasability(tCOMBAT[a][i], "360 Vision") then
          if next(tCOMBAT[b])~=nil then
            e = geteffectbyname("Hidden")
            for j = 1, #tCOMBAT[b], 1 do
              if tCOMBAT[b][j].Effect[e]>0 then
                addfloatytext(tCOMBAT[a][i], "+360 Vision+")
                addfloatytext(tCOMBAT[b][j], "-Revealed-")
                tCOMBAT[b][j].Effect[e]=0
              end
            end
          end
        elseif gethasability(tCOMBAT[a][i], "Echolocation") then
          if next(tCOMBAT[b])~=nil then
            e = geteffectbyname("Hidden")
            for j = 1, #tCOMBAT[b], 1 do
              if tCOMBAT[b][j].Effect[e]>0 then
                addfloatytext(tCOMBAT[a][i], "+Echolocation+")
                addfloatytext(tCOMBAT[b][j], "-Revealed-")
                tCOMBAT[b][j].Effect[e]=0
              end
            end
          end
        else end
        if gethasability(tCOMBAT[a][i], "Kick") and not gethaseffect(tCOMBAT[a][i], "Immobile") and not gethaseffect(tCOMBAT[a][i], "Limp") and not gethaseffect(tCOMBAT[a][i], "Passive") then
          if tCOMBAT[a][i].GX>1 then
            pl, index = getsquare(tCOMBAT[a][i].GX-1, tCOMBAT[a][i].GY)
            if pl==b then damage(tCOMBAT[b][index], 1) end
          end
          if tCOMBAT[a][i].GX<tGAME.Gridsize then
            pl, index = getsquare(tCOMBAT[a][i].GX+1, tCOMBAT[a][i].GY)
            if pl==b then damage(tCOMBAT[b][index], 1) end
          end
          if tCOMBAT[a][i].GY>1 then
            pl, index = getsquare(tCOMBAT[a][i].GX, tCOMBAT[a][i].GY-1)
            if pl==b then damage(tCOMBAT[b][index], 1) end
          end
          if tCOMBAT[a][i].GY<tGAME.Gridsize then
            pl, index = getsquare(tCOMBAT[a][i].GX, tCOMBAT[a][i].GY+1)
            if pl==b then damage(tCOMBAT[b][index], 1) end
          end
        end
      else
        table.insert(tGRAVE[a], deepcopy(tCOMBAT[a][i]))
        table.remove(tCOMBAT[a], i)
      end
    end
  end
  if next(tDECK[b])~=nil then
    for i = #tDECK[b], 1, -1 do
      for j = 1, #tEFFECTS, 1 do
        if tDECK[b][i].Effect[j]<100 then tDECK[b][i].Effect[j]=math.max(tDECK[b][i].Effect[j]-1,0) end
      end
    end
  end
  if next(tCOMBAT[b])~=nil then
    for i = #tCOMBAT[b], 1, -1 do
      for j = 1, #tEFFECTS, 1 do
        if tCOMBAT[b][i].Effect[j]<100 then tCOMBAT[b][i].Effect[j]=math.max(tCOMBAT[b][i].Effect[j]-1,0) end
      end
      if gethasability(tCOMBAT[b][i], "Nocturnal") then
        if gDAYNIGHT>=6 then 
          addfloatytext(tCOMBAT[b][i], "+Nocturnal+")          
          seteffect("Nighttime", 1, "Combat", b, i, "Max")
        else 
          addfloatytext(tCOMBAT[b][i], "-Nocturnal-")          
          seteffect("Daytime", 1, "Combat", b, i, "Max")
        end
      end
      if gethasability(tCOMBAT[b][i], "Scary to Mammals") then
        if tCOMBAT[b][i].GX>1 then
          pl, index = getsquare(tCOMBAT[b][i].GX-1, tCOMBAT[b][i].GY)
          if pl==a and tCOMBAT[pl][index].Class=="Mammal" and (math.random(100)<=50 or gethasability(tCOMBAT[pl][index], "Fearful")) and (tCOMBAT[pl][index].Size=="Medium" or tCOMBAT[pl][index].Size=="Small" or tCOMBAT[pl][index].Size=="Tiny") then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Scary+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
        if tCOMBAT[b][i].GX<tGAME.Gridsize then
          pl, index = getsquare(tCOMBAT[b][i].GX+1, tCOMBAT[b][i].GY)
          if pl==a and tCOMBAT[pl][index].Class=="Mammal" and (math.random(100)<=50 or gethasability(tCOMBAT[pl][index], "Fearful")) and (tCOMBAT[pl][index].Size=="Medium" or tCOMBAT[pl][index].Size=="Small" or tCOMBAT[pl][index].Size=="Tiny") then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Scary+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
        if tCOMBAT[b][i].GY>1 then
          pl, index = getsquare(tCOMBAT[b][i].GX, tCOMBAT[b][i].GY-1)
          if pl==a and tCOMBAT[pl][index].Class=="Mammal" and (math.random(100)<=50 or gethasability(tCOMBAT[pl][index], "Fearful")) and (tCOMBAT[pl][index].Size=="Medium" or tCOMBAT[pl][index].Size=="Small" or tCOMBAT[pl][index].Size=="Tiny") then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Scary+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
        if tCOMBAT[b][i].GY<tGAME.Gridsize then
          pl, index = getsquare(tCOMBAT[b][i].GX, tCOMBAT[b][i].GY+1)
          if pl==a and tCOMBAT[pl][index].Class=="Mammal" and (math.random(100)<=50 or gethasability(tCOMBAT[pl][index], "Fearful")) and (tCOMBAT[pl][index].Size=="Medium" or tCOMBAT[pl][index].Size=="Small" or tCOMBAT[pl][index].Size=="Tiny") then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Scary+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
      end
      if gethasability(tCOMBAT[b][i], "Scary to Small Creatures") then
        if tCOMBAT[b][i].GX>1 then
          pl, index = getsquare(tCOMBAT[b][i].GX-1, tCOMBAT[b][i].GY)
          if pl==a and (math.random(100)<=50 or gethasability(tCOMBAT[pl][index], "Fearful")) and tCOMBAT[pl][index].Size=="Small" then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Barks+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
        if tCOMBAT[b][i].GX<tGAME.Gridsize then
          pl, index = getsquare(tCOMBAT[b][i].GX+1, tCOMBAT[b][i].GY)
          if pl==a and (math.random(100)<=50 or gethasability(tCOMBAT[pl][index], "Fearful")) and tCOMBAT[pl][index].Size=="Small" then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Barks+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
        if tCOMBAT[b][i].GY>1 then
          pl, index = getsquare(tCOMBAT[b][i].GX, tCOMBAT[b][i].GY-1)
          if pl==a and (math.random(100)<=50 or gethasability(tCOMBAT[pl][index], "Fearful")) and tCOMBAT[pl][index].Size=="Small" then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Barks+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
        if tCOMBAT[b][i].GY<tGAME.Gridsize then
          pl, index = getsquare(tCOMBAT[b][i].GX, tCOMBAT[b][i].GY+1)
          if pl==a and (math.random(100)<=50 or gethasability(tCOMBAT[pl][index], "Fearful")) and tCOMBAT[pl][index].Size=="Small" then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Barks+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
      end
      if gethasability(tCOMBAT[b][i], "Mesmerise") then
        if tCOMBAT[b][i].GX>1 then
          pl, index = getsquare(tCOMBAT[b][i].GX-1, tCOMBAT[b][i].GY)
          if pl==a and math.random(100)<=33 then
            seteffect("Immobile", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Mesmerise+")
            addfloatytext(tCOMBAT[pl][index], "-Distracted-")
          end
        end
        if tCOMBAT[b][i].GX<tGAME.Gridsize then
          pl, index = getsquare(tCOMBAT[b][i].GX+1, tCOMBAT[b][i].GY)
          if pl==a and math.random(100)<=33 then
            seteffect("Immobile", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Mesmerise+")
            addfloatytext(tCOMBAT[pl][index], "-Distracted-")
          end
        end
        if tCOMBAT[b][i].GY>1 then
          pl, index = getsquare(tCOMBAT[b][i].GX, tCOMBAT[b][i].GY-1)
          if pl==a and math.random(100)<=33 then
            seteffect("Immobile", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Mesmerise+")
            addfloatytext(tCOMBAT[pl][index], "-Distracted-")
          end
        end
        if tCOMBAT[b][i].GY<tGAME.Gridsize then
          pl, index = getsquare(tCOMBAT[b][i].GX, tCOMBAT[b][i].GY+1)
          if pl==a and math.random(100)<=33 then
            seteffect("Immobile", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Mesmerise+")
            addfloatytext(tCOMBAT[pl][index], "-Distracted-")
          end
        end
      end
      if gethasability(tCOMBAT[b][i], "Intimidate") then
        if tCOMBAT[b][i].GX>1 then
          pl, index = getsquare(tCOMBAT[b][i].GX-1, tCOMBAT[b][i].GY)
          if pl==a and tCOMBAT[pl][index].Class=="Mammal" and (tCOMBAT[pl][index].Size=="Medium" or tCOMBAT[pl][index].Size=="Small" or tCOMBAT[pl][index].Size=="Tiny") then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Intimidate+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
        if tCOMBAT[b][i].GX<tGAME.Gridsize then
          pl, index = getsquare(tCOMBAT[b][i].GX+1, tCOMBAT[b][i].GY)
          if pl==a and tCOMBAT[pl][index].Class=="Mammal" and (tCOMBAT[pl][index].Size=="Medium" or tCOMBAT[pl][index].Size=="Small" or tCOMBAT[pl][index].Size=="Tiny") then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Intimidate+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
        if tCOMBAT[b][i].GY>1 then
          pl, index = getsquare(tCOMBAT[b][i].GX, tCOMBAT[b][i].GY-1)
          if pl==a and tCOMBAT[pl][index].Class=="Mammal" and (tCOMBAT[pl][index].Size=="Medium" or tCOMBAT[pl][index].Size=="Small" or tCOMBAT[pl][index].Size=="Tiny") then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Intimidate+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
        if tCOMBAT[b][i].GY<tGAME.Gridsize then
          pl, index = getsquare(tCOMBAT[b][i].GX, tCOMBAT[b][i].GY+1)
          if pl==a and tCOMBAT[pl][index].Class=="Mammal" and (tCOMBAT[pl][index].Size=="Medium" or tCOMBAT[pl][index].Size=="Small" or tCOMBAT[pl][index].Size=="Tiny") then
            seteffect("Passive", 1, "Combat", pl, index, "Max")
            addfloatytext(tCOMBAT[b][i], "+Intimidate+")
            addfloatytext(tCOMBAT[pl][index], "-Fear-")
          end
        end
      end
    end
  end
  if tCONTROL[gTURN]=="AI" then
    AIlastmove = love.timer.getTime()
    AIbest_score = scoregrid(gTURN)
    AIbest_move = {}
    AIloop = 0
  end
  -- Check for skip turns
  j = true
  for i = 1, tGAME.Gridsize, 1 do
    if tZONE[i]~=a then j = false end
  end
  if next(tDECK[a])==nil and next(tCOMBAT[a])==nil then gWAIT = 1
  elseif j==true and next(tDECK[b])==nil and next(tCOMBAT[b])==nil then gWAIT = 1
  else end
end

function processresult()  -- Process who won/lost
  local i, new
  local deck, list = {}, {}
  scorezones()
  gDISPLAYHAND=false
  xBOARD = 400
  arrangecombat()
  if tGAME.Score[1]>tGAME.Score[2] then
    tGAME.Winner, tGAME.Loser = 1, 2
    if tCONTROL[2]=="AI" then
      -- Update win percentage
      tPLAYER.WIN = tPLAYER.WIN + 1
      -- Has the player unlocked a new card?
      deck = loaddeck("ai/deck"..tostring((3*(gLEVEL-1))+gDIFFICULTY)..".txt")
      if next(deck)~=nil then
        for i = 1, #deck, 1 do
          if not getiscardunlocked(deck[i]) then table.insert(list, deck[i].Name) end
        end
      end
      if next(list)~=nil then
        new = math.random(#list)
        for i = 1, #tBLUEPRINT, 1 do
          if tBLUEPRINT[i].Name==list[new] then gNEWCARD = i end
        end
      end
      if gNEWCARD~=0 then table.insert(tUNLOCKED, tBLUEPRINT[gNEWCARD].Name) end
      -- Has the player unlocked a new level?
      if gLEVEL==tPLAYER.LEVEL and tPLAYER.LEVEL<11 then
        gNEWLEVEL, tPLAYER.LEVEL = tPLAYER.LEVEL+1, tPLAYER.LEVEL+1
      end
      -- Gold reward for level
      gNEWGOLD = tGAME.Prize
      tPLAYER.GOLD = tPLAYER.GOLD + gNEWGOLD
      writesinglelist("unlocked.txt", tUNLOCKED)
      writesingletable("player.txt", tPLAYER)
    end
  elseif tGAME.Score[2]>tGAME.Score[1] then
    tGAME.Winner, tGAME.Loser = 2, 1
  else end
  gGAMEMODE = "score"
  musicplay(3)
end

function love.update()  -- Runs every frame
  updategeneral()
  if gGAMEMODE=="game" then updategame() end
  musicupdate()
  collectgarbage()
  if love.timer.getTime()-gTIMER<=(1/tSETTINGS.FPS) then love.timer.sleep((1/tSETTINGS.FPS)-(love.timer.getTime()-gTIMER)) end  -- frame rate
  gTIMER = love.timer.getTime()  -- new timer
end

function updategeneral()  -- Updates general variables, fade and flash
  if love.mouse.isDown(1) then gMOUSEGRAB = gMOUSEGRAB + 1 end
  if gFADE > 0 then  -- fade in graphics
    gFADE = gFADE - gFADESPEED
    if gFADE < 0 then gFADE = 0 end
  end
  gFLASH = gFLASH + gFLASH_z  -- flash counter
  if gFLASH > 255 then  -- this affects everything that flashes (same speed)
    gFLASH = gFLASH_max
    gFLASH_z = 0 - gFLASH_z
  elseif gFLASH < gFLASH_min then
    gFLASH = gFLASH_min
    gFLASH_z = 0 - gFLASH_z
  else end
end

function updategame()  -- Updates game variables every tick
  local i, j
  for j = 1, 2, 1 do
    if next(tDECK[j])~=nil then
      for i = 1, #tDECK[j], 1 do
        if tDECK[j][i].Animate == true then animatecard(tDECK[j][i]) end
        if next(tDECK[j][i].FloatyText)~=nil then animatetext(tDECK[j][i]) end
      end
    end
    if next(tCOMBAT[j])~=nil then
      for i = 1, #tCOMBAT[j], 1 do
        if tCOMBAT[j][i].Animate == true then animatecard(tCOMBAT[j][i]) end
        if next(tCOMBAT[j][i].FloatyText)~=nil then animatetext(tCOMBAT[j][i]) end
      end
    end
    if next(tGRAVE[j])~=nil then
      for i = 1, #tGRAVE[j], 1 do
        if tGRAVE[j][i].Animate == true then animatecard(tGRAVE[j][i]) end
        if next(tGRAVE[j][i].FloatyText)~=nil then animatetext(tGRAVE[j][i]) end
      end
    end
  end
  if gWAIT > 0 then 
    gWAIT = gWAIT - 1
    if gWAIT == 0 then
      if gROUND==tGAME.Rounds then processresult()
      else 
        newturn()
        if tCONTROL[2]=="Player" then gGAMEMODE = "newturn" end
      end
    end
  elseif tCONTROL[gTURN] == "AI" and gWAIT==0 then AI()  -- Call the A.I.
  else end
end

function love.draw()  -- Draw everything in, based on gamemode
  love.graphics.scale(gRATIOX, gRATIOY)
  if _G["draw"..gGAMEMODE]~=nil then _G["draw"..gGAMEMODE]() end
  if gFADE > 0 then 
    love.graphics.setColor(0, 0, 0, gFADE)
    love.graphics.rectangle("fill", 0, 0, 800, 600)
  else end
end

function drawshadedrectangle(x, y, width, height, r, g, b)  -- Draw a semi-transparent rectangle
  r, g, b = r or 255, g or 255, b or 255
  love.graphics.setColor(r, g, b, 180)
  love.graphics.rectangle("fill", x, y, width, height)
  love.graphics.setColor(r, g, b, 255)
  love.graphics.rectangle("line", x, y, width, height)
end

function drawthickrectangle(x, y, width, height, r, g, b, a)  -- Draw a thick edged line rectangle
  local i
  r, g, b, a = r or 255, g or 255, b or 255, a or 255
  love.graphics.setColor(r, g, b, a)
  for i = 0, 2, 1 do
    love.graphics.rectangle("line", x+i, y+i, width-(i*2), height-(i*2))
  end
end

function drawshadowtext(text, x, y, limit, align, font, r, g, b, a, hr, hg, hb, ha)  -- Draw text with a shadow
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local width, wrappedtext = font:getWrap(text, limit)
  local height = font:getHeight()
  local o
  font = font or fDEFAULT
  r, g, b, a = r or 255, g or 255, b or 255, a or 255
  hr, hg, hb, ha = hr or r, hg or g, hb or b, ha or a
  if font==fTITLE then o = 3
  elseif font==fHEADING then o = 2
  else o = 1 end
  love.graphics.setFont(font)
  love.graphics.setColor(0, 0, 0, a)
  love.graphics.printf(text, x+o, y+o, limit, align)
  if mx>x and mx<x+limit and my>y and my<y+(#wrappedtext*height) then 
    love.graphics.setColor(hr, hg, hb, ha)
    love.graphics.printf(text, x, y, limit, align)
  else
    love.graphics.setColor(r, g, b, a)
    love.graphics.printf(text, x, y, limit, align)
  end
end

function drawmenu()  -- Draw the main menu
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[12], 0, 0)
  drawshadowtext("Menu", 330, 75, 140, "center", fTITLE, 255, 215, 0)
  drawshadowtext("Campaign", 300, 200, 200, "center", fHEADING, 255, 215, 0, 255, 255, 255, 255, 255)
  drawshadowtext("Pass and Play", 300, 250, 200, "center", fHEADING, 255, 215, 0, 255, 255, 255, 255, 255)
  drawshadowtext("Shop", 300, 300, 200, "center", fHEADING, 255, 215, 0, 255, 255, 255, 255, 255)
  drawshadowtext("Settings", 300, 350, 200, "center", fHEADING, 255, 215, 0, 255, 255, 255, 255, 255)
  drawshadowtext("Quit", 300, 400, 200, "center", fHEADING, 255, 215, 0, 255, 255, 255, 255, 255)
end

function drawmap()  -- Draw the campaign map
  local i, easy, med, hard
  if gSUBSELECT~=0 then
    easy = readsingletable("ai/game"..tostring((3*(gSELECT-1))+1)..".txt")
    med = readsingletable("ai/game"..tostring((3*(gSELECT-1))+2)..".txt")
    hard = readsingletable("ai/game"..tostring((3*(gSELECT-1))+3)..".txt")
  end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iMAP[tPLAYER.LEVEL], 0, 0)
  drawshadowtext("Back", 720, 560, 70, "center", fHEADING, 255, 215, 0, 255, 255, 255, 255, 255)
  for i = 1, tPLAYER.LEVEL, 1 do
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.circle("fill", tMAP[i].X, tMAP[i].Y, 6)
    if i == gSELECT then love.graphics.circle("line", tMAP[i].X, tMAP[i].Y, 6+(gFLASH/50)) end
  end
  if gSUBSELECT~=0 then
    drawshadedrectangle(0, 0, 800, 100, 0, 0, 0)
    drawshadedrectangle(0, 100, 266, 400, 0, 255, 0)
    drawshadedrectangle(267, 100, 266, 400, 255, 165, 0)
    drawshadedrectangle(534, 100, 266, 400, 255, 0, 0)
    drawshadedrectangle(0, 500, 800, 100, 0, 0, 0)
    drawshadowtext(tMAP[gSELECT].Name, 0, 20, 800, "center", fTITLE)
    drawshadowtext("EASY", 0, 120, 266, "center", fTITLE)
    drawshadowtext("Size: "..easy.Gridsize.."x"..easy.Gridsize, 0, 180, 266, "center", fTITLE)
    drawshadowtext("Pieces: "..easy.Pieces, 0, 240, 266, "center", fTITLE)
    drawshadowtext("Turns: "..easy.Rounds/2, 0, 300, 266, "center", fTITLE)
    drawshadowtext("Win:", 0, 400, 133, "right", fTITLE)
    drawshadowtext("MEDIUM", 267, 120, 266, "center", fTITLE)
    drawshadowtext("Size: "..med.Gridsize.."x"..med.Gridsize, 267, 180, 266, "center", fTITLE)
    drawshadowtext("Pieces: "..med.Pieces, 267, 240, 266, "center", fTITLE)
    drawshadowtext("Turns: "..med.Rounds/2, 267, 300, 266, "center", fTITLE)
    drawshadowtext("Win:", 267, 400, 133, "right", fTITLE)
    drawshadowtext("HARD", 534, 120, 266, "center", fTITLE)
    drawshadowtext("Size: "..hard.Gridsize.."x"..hard.Gridsize, 534, 180, 266, "center", fTITLE)
    drawshadowtext("Pieces: "..hard.Pieces, 534, 240, 266, "center", fTITLE)
    drawshadowtext("Turns: "..hard.Rounds/2, 534, 300, 266, "center", fTITLE)
    drawshadowtext("Win:", 534, 400, 133, "right", fTITLE)
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.draw(iICON[31], 140, 384)
    love.graphics.draw(iICON[31], 407, 384)
    love.graphics.draw(iICON[31], 674, 384)
    drawshadowtext("$"..easy.Prize, 140, 400, 80, "center", fTITLE)
    drawshadowtext("$"..med.Prize, 407, 400, 80, "center", fTITLE)
    drawshadowtext("$"..hard.Prize, 674, 400, 80, "center", fTITLE)
  end
end

function drawselectdeck()  -- Draw the deck selection screen
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local list = {} 
  local r, g, b = getcolours(gTURN)
  local i, x, y
  if gSELECT~=0 and gSUBSELECT~=0 then list = getlistofcards(tCOLLECTION, tCLASS[gSUBSELECT], tTIER[gSELECT], "Alphabetical")
  elseif gSELECT~=0 then list = getlistofcards(tCOLLECTION, "All", tTIER[gSELECT], "Alphabetical")
  elseif gSUBSELECT~=0 then list = getlistofcards(tCOLLECTION, tCLASS[gSUBSELECT], 0, "Tier")
  else list = getlistofcards(tCOLLECTION, "All", 0, "Tier") end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[12], 0, 0)
  love.graphics.draw(iICON[6], 0, 0)
  drawshadedrectangle(250, 10, 300, 50, r, g, b)
  if gTURN==1 then drawshadowtext(tSETTINGS.Player1, 255, 15, 290, "center", fHEADING)
  else drawshadowtext(tSETTINGS.Player2, 255, 15, 290, "center", fHEADING) end
  if #tDECK[gTURN]<math.min(tGAME.Pieces,#tCOLLECTION) then
    drawshadowtext("Choose "..math.min(tGAME.Pieces,#tCOLLECTION)-#tDECK[gTURN].." cards", 565, 15, 220, "center", fHEADING)
  else
    drawshadedrectangle(560, 10, 230, 50)
    drawshadowtext("Ready!", 565, 15, 220, "center", fHEADING, 255, 255, 255, 255, 255, 215, 0, 255)
  end
  if next(tTIER)~=nil then
    if gSELECT==0 then drawshadedrectangle(25, 70, 125, 40)
    else drawshadedrectangle(25, 70, 125, 40, 160, 160, 160) end
    drawshadowtext("All", 30, 70, 115, "center", fHEADING, 255, 255, 255, 255, 255, 215, 0, 255)
    for i = 1, #tTIER, 1 do
      if gSELECT==i then drawshadedrectangle(25, (i*50)+70, 125, 40)
      else drawshadedrectangle(25, (i*50)+70, 125, 40, 160, 160, 160) end
      drawshadowtext("Tier "..tTIER[i], 30, (i*50)+70, 115, "center", fHEADING, 255, 255, 255, 255, 255, 215, 0, 255)
    end
  end
  if next(tCLASS)~=nil then
    if gSUBSELECT==0 then drawshadedrectangle(650, 70, 125, 40)
    else drawshadedrectangle(650, 70, 125, 40, 160, 160, 160) end
    drawshadowtext("All", 655, 70, 115, "center", fHEADING, 255, 255, 255, 255, 255, 215, 0, 255)
    for i = 1, #tCLASS, 1 do
      if gSUBSELECT==i then drawshadedrectangle(650, (i*50)+70, 125, 40)
      else drawshadedrectangle(650, (i*50)+70, 125, 40, 160, 160, 160) end
      drawshadowtext(tCLASS[i], 655, (i*50)+70, 115, "center", fHEADING, 255, 255, 255, 255, 255, 215, 0, 255)
    end
  end
  if next(list)~=nil then
    love.graphics.setColor(255, 255, 255, 255)
    if gPAGE>1 then love.graphics.draw(iICON[4], 54, 522) end
    if ((gPAGE-1)*20)+20<#list then love.graphics.draw(iICON[5], 678, 522) end
    x, y = 220, 120
    for i = ((gPAGE-1)*20)+1, math.min(((gPAGE-1)*20)+20, #list), 1 do
      if getisnameinlist(list[i], tDECK[gTURN]) then drawcardicon(list[i], x, y, 100, 100, 255, 255, 255, 255)
      else drawcardicon(list[i], x, y, 100, 100, 100, 100, 100, 255) end
      x = x + 120
      if x>580 then
        x = 220
        y = y + 105
      end
    end
    if gZOOMED~=true then 
      x, y = 220, 120
      for i = ((gPAGE-1)*20)+1, math.min(((gPAGE-1)*20)+20, #list), 1 do
        if mx>=x-50 and mx<=x+50 and my>=y-50 and my<=y+50 then
          drawthickrectangle(x-50, y-50, 100, 100, r, g, b)
          drawshadedrectangle(mx-80, my+30, 160, 25, 100, 100, 100)
          drawshadowtext(list[i].Name, mx-80, my+30, 160, "center", fDEFAULT)
        end
        x = x + 120
        if x>580 then
          x = 220
          y = y + 105
        end
      end
    end
    if gZOOMED==true then
      drawshadedrectangle(0, 0, 800, 600, 0, 0, 0)
      if getisnameinlist(tCLICK.Card, tDECK[gTURN]) then
        drawshadedrectangle(250, 520, 300, 60, 255, 255, 255)
        drawshadowtext("Selected", 255, 525, 290, "center", fTITLE)
      else 
        drawshadedrectangle(250, 520, 300, 60, 100, 100, 100)
        drawshadowtext("Not Selected", 255, 525, 290, "center", fTITLE)
      end
      drawcard(tCLICK.Card, 400, 300, 300, 420, r, g, b)
    end
  end
end

function drawgallery()  -- Draw the gallery screen
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local list = {} 
  local i, x, y
  if gSELECT~=0 and gSUBSELECT~=0 then list = getlistofcards(tBLUEPRINT, tCLASS[gSUBSELECT], tTIER[gSELECT], "Alphabetical")
  elseif gSELECT~=0 then list = getlistofcards(tBLUEPRINT, "All", tTIER[gSELECT], "Alphabetical")
  elseif gSUBSELECT~=0 then list = getlistofcards(tBLUEPRINT, tCLASS[gSUBSELECT], 0, "Tier")
  else list = getlistofcards(tBLUEPRINT, "All", 0, "Tier") end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[12], 0, 0)
  love.graphics.draw(iICON[6], 0, 0)
  love.graphics.draw(iICON[31], 40, 0, 0, 0.5, 0.5)
  drawshadedrectangle(250, 10, 300, 50)
  drawshadowtext("Unlocked "..#tUNLOCKED.." / "..#tBLUEPRINT, 255, 15, 290, "center", fHEADING)
  drawshadowtext("$"..tPLAYER.GOLD, 80, 6, 170, "left", fHEADING, 255, 215, 0, 255)
  if next(tTIER)~=nil then
    if gSELECT==0 then drawshadedrectangle(25, 70, 125, 40)
    else drawshadedrectangle(25, 70, 125, 40, 160, 160, 160) end
    drawshadowtext("All", 30, 70, 115, "center", fHEADING, 255, 255, 255, 255, 255, 215, 0, 255)
    for i = 1, #tTIER, 1 do
      if gSELECT==i then drawshadedrectangle(25, (i*50)+70, 125, 40)
      else drawshadedrectangle(25, (i*50)+70, 125, 40, 160, 160, 160) end
      drawshadowtext("Tier "..tTIER[i], 30, (i*50)+70, 115, "center", fHEADING, 255, 255, 255, 255, 255, 215, 0, 255)
    end
  end
  if next(tCLASS)~=nil then
    if gSUBSELECT==0 then drawshadedrectangle(650, 20, 125, 40)
    else drawshadedrectangle(650, 20, 125, 40, 160, 160, 160) end
    drawshadowtext("All", 655, 20, 115, "center", fHEADING, 255, 255, 255, 255, 255, 215, 0, 255)
    for i = 1, #tCLASS, 1 do
      if gSUBSELECT==i then drawshadedrectangle(650, (i*50)+20, 125, 40)
      else drawshadedrectangle(650, (i*50)+20, 125, 40, 160, 160, 160) end
      drawshadowtext(tCLASS[i], 655, (i*50)+20, 115, "center", fHEADING, 255, 255, 255, 255, 255, 215, 0, 255)
    end
  end
  if next(list)~=nil then
    love.graphics.setColor(255, 255, 255, 255)
    if gPAGE>1 then love.graphics.draw(iICON[4], 54, 522) end
    if ((gPAGE-1)*20)+20<#list then love.graphics.draw(iICON[5], 678, 522) end
    x, y = 220, 120
    for i = ((gPAGE-1)*20)+1, math.min(((gPAGE-1)*20)+20, #list), 1 do
      drawcardicon(list[i], x, y, 100, 100, 255, 255, 255, 255)
      if getisnameinlist(list[i], tCOLLECTION) then -- Do nothing, they own it already!
      else
        if getiscardunlocked(list[i]) then
          love.graphics.setColor(255, 255, 255, 255)
          love.graphics.draw(iICON[31], x-40, y-40)
          drawshadowtext("$"..list[i].Cost, x-40, y-14, 80, "center", fHEADING)
        else 
          drawshadedrectangle(x-50, y-50, 100, 100, 0, 0, 0)
          love.graphics.setColor(100, 100, 100, 255)
          love.graphics.draw(iICON[30], x-40, y-40)
        end
      end
      x = x + 120
      if x>580 then
        x = 220
        y = y + 105
      end
    end
    if gZOOMED~=true then 
      x, y = 220, 120
      for i = ((gPAGE-1)*20)+1, math.min(((gPAGE-1)*20)+20, #list), 1 do
        if mx>=x-50 and mx<=x+50 and my>=y-50 and my<=y+50 then
          drawthickrectangle(x-50, y-50, 100, 100)
          drawshadedrectangle(mx-80, my+30, 160, 25, 100, 100, 100)
          drawshadowtext(list[i].Name, mx-80, my+30, 160, "center", fDEFAULT)
        end
        x = x + 120
        if x>580 then
          x = 220
          y = y + 105
        end
      end
    end
    if gZOOMED==true then
      drawshadedrectangle(0, 0, 800, 600, 0, 0, 0)
      if getisnameinlist(tCLICK.Card, tCOLLECTION) then
        drawshadedrectangle(250, 520, 300, 60, 255, 255, 255)
        drawshadowtext("Owned", 255, 525, 290, "center", fTITLE)
      elseif getiscardunlocked(tCLICK.Card) then 
        if tPLAYER.GOLD>=tCLICK.Card.Cost then
          drawshadedrectangle(250, 520, 300, 60, 255, 255, 255)
          drawshadowtext("Purchase for $"..tCLICK.Card.Cost, 255, 525, 290, "center", fTITLE, 255, 255, 255, 255, 255, 215, 0, 255)
        else
          drawshadedrectangle(250, 520, 300, 60, 100, 100, 100)
          drawshadowtext("Not enough gold", 255, 525, 290, "center", fTITLE)
        end
      else
        drawshadedrectangle(250, 520, 300, 60, 100, 100, 100)
        drawshadowtext("Not yet unlocked", 255, 525, 290, "center", fTITLE)
      end
      drawcard(tCLICK.Card, 400, 300, 300, 420)
    end
  end
end

function drawcardicon(card, x, y, w, h, r, g, b, a)  -- Draw card image square
  local power = getpower(card)
  local health = gethealth(card)
  r, g, b, a = r or 255, g or 255, b or 255, a or 255
  love.graphics.setFont(fHEADING)
  love.graphics.setColor(r, g, b, a)
  if card.Pic~=nil then love.graphics.draw(iIMAGE[card.Pic], x, y, 0, w/iIMAGE[card.Pic]:getWidth(), h/iIMAGE[card.Pic]:getHeight(), iIMAGE[card.Pic]:getWidth()/2, iIMAGE[card.Pic]:getHeight()/2)
  else love.graphics.draw(iUNKNOWN, x, y, 0, w/iUNKNOWN:getWidth(), h/iUNKNOWN:getHeight(), iUNKNOWN:getWidth()/2, iUNKNOWN:getHeight()/2) end
  love.graphics.draw(iICON[8], x+(w*0.35), y+(h*0.35), 0, (w*0.3)/iICON[8]:getWidth(), (h*0.3)/iICON[8]:getHeight(), iICON[8]:getWidth()/2, iICON[8]:getHeight()/2)
  drawshadowtext(power, x+(w*0.35)+((w*0.3)/(2*iICON[8]:getWidth()))-(w*0.3), y+(h*0.35)-16, w*0.6, "center", fHEADING, r, g, b, a)
  love.graphics.setColor(r, g*0.8, 0, a)
  love.graphics.draw(iICON[11], x-(w*0.35), y-(h*0.35), 0, (w*0.3)/iICON[11]:getWidth(), (h*0.3)/iICON[11]:getHeight(), iICON[11]:getWidth()/2, iICON[11]:getHeight()/2)
  drawshadowtext(card.Tier, x-(w*0.5), y-(h*0.35)-16, w*0.3, "center", fHEADING, r, g, b, a)
  if health>card.MaxHealth*0.7 then love.graphics.setColor(0, g, 0, a)
  elseif health>card.MaxHealth*0.4 then love.graphics.setColor(r, g/2, 0, a)
  else love.graphics.setColor(r, 0, 0, a) end
  love.graphics.draw(iICON[9], x-(w*0.35), y+(h*0.35), 0, (w*0.3)/iICON[9]:getWidth(), (h*0.3)/iICON[9]:getHeight(), iICON[9]:getWidth()/2, iICON[9]:getHeight()/2)
  drawshadowtext(health, x-(w*0.35)-((w*0.3)/(2*iICON[9]:getWidth()))-(w*0.3), y+(h*0.35)-16, w*0.6, "center", fHEADING, r, g, b, a)
  if gGAMEMODE=="selectdeck" then
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.draw(iICON[10], x+(w*0.35), y-(h*0.35), 0, (w*0.3)/iICON[10]:getWidth(), (h*0.3)/iICON[10]:getHeight(), iICON[10]:getWidth()/2, iICON[10]:getHeight()/2)
  end
end

function drawcard(card, x, y, width, height, r, g, b, alpha)  -- Draw the full card
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local h, a, p = gethealth(card), getpower(card), getpatternbyname(card.Pattern)
  local sx, sy, i, j
  width, height = width or 300, height or 420
  r, g, b, alpha = r or 255, g or 255, b or 255, alpha or 255
  sx, sy = width/300, height/420  
  love.graphics.setColor(r, g, b, alpha)
  love.graphics.draw(iCARD, x, y, 0, sx, sy, 150, 210)                                      -- Draw card front in rgb colour
  love.graphics.setColor(255, 255, 255, alpha)
  love.graphics.draw(iIMAGE[card.Pic], x, y, 0, sx, sy, 80, 155)                            -- Draw creature pic
  love.graphics.draw(iSQUARE, x, y, 0, sx, sy, 80, 155)                                     -- Draw square around pic
  love.graphics.draw(iICON[8], x, y, 0, sx, sy, -78, -132)                                  -- Draw claw icon (attack)
  if p~=0 then love.graphics.draw(iICON[tPATTERN[p].Pic], x, y, 0, sx, sy, 40, -130) end    -- Draw movement pattern icon
  love.graphics.setColor(255, 215, 0, 255)
  love.graphics.draw(iICON[11], x, y, 0, sx, sy, 144, 204)                                  -- Draw tier star
  love.graphics.setFont(fTITLE)
  love.graphics.setColor(0, 0, 0, alpha)
  love.graphics.printf(a, x, y, 68, "center", 0, sx, sy, -78, -156)                         -- Print attack (black)
  love.graphics.printf(card.Tier, x, y, 60, "center", 0, sx, sy, 144, 190)                  -- Print tier (black)
  love.graphics.setColor(255, 255, 255, alpha)
  love.graphics.printf(a, x, y, 68, "center", 0, sx, sy, -80, -158)                         -- Print attack (white)
  love.graphics.printf(card.Tier, x, y, 60, "center", 0, sx, sy, 146, 192)                  -- Print tier (white)
  if h > card.MaxHealth*0.7 then love.graphics.setColor(0, 255, 0, alpha)                   -- 70%+ health=green
  elseif h > card.MaxHealth*0.4 then love.graphics.setColor(255, 150, 0, alpha)             -- 40%+ health=orange
  else love.graphics.setColor(255, 0, 0, alpha) end                                         -- <40% health=red
  love.graphics.draw(iICON[9], x, y, 0, sx, sy, 146, -146)                                  -- Draw health heart in colour
  love.graphics.setFont(fHEADING)
  love.graphics.setColor(0, 0, 0, alpha)
  love.graphics.printf(card.Name, x, y, 230, "left", 0, sx, sy, 76, 208)                    -- Print name of card (black)
  love.graphics.printf(h, x, y, 50, "center", 0, sx, sy, 149, -148)                         -- Print health (black)
  love.graphics.printf("/", x, y, 60, "center", 0, sx, sy, 143, -156)                       -- Out of (black)
  love.graphics.printf(card.MaxHealth, x, y, 50, "center", 0, sx, sy, 124, -164)            -- Maxhealth (black)
  love.graphics.setColor(255, 255, 255, alpha)
  love.graphics.printf(card.Name, x, y, 230, "left", 0, sx, sy, 78, 210)                    -- Print name of card (white)
  love.graphics.printf(h, x, y, 50, "center", 0, sx, sy, 151, -146)                         -- Print health (white)
  love.graphics.printf("/", x, y, 60, "center", 0, sx, sy, 144, -155)                       -- Out of (white)
  love.graphics.printf(card.MaxHealth, x, y, 50, "center", 0, sx, sy, 126, -162)            -- Maxhealth (white)
  love.graphics.setFont(fDEFAULT)
  love.graphics.setColor(0, 0, 0, alpha)
  love.graphics.printf(card.Class..", "..card.Size, x, y, 160, "left", 0, sx, sy, 76, 181)  -- Print card class & size (black)
  love.graphics.printf(card.Tag, x, y, 270, "left", 0, sx, sy, 134, -8)                     -- Print card tag (black)
  for i = 1, 3, 1 do
    if card["Spec"..i]~=nil then 
      love.graphics.printf(card["Spec"..i], x, y, 270, "left", 0, sx, sy, 134, -56-(i*20))  -- Print special abilities (black)
    end
  end
  love.graphics.setColor(255, 255, 255, alpha)
  love.graphics.printf(card.Class..", "..card.Size, x, y, 160, "left", 0, sx, sy, 77, 182)  -- Print card class & size (white)
  love.graphics.printf(card.Tag, x, y, 270, "left", 0, sx, sy, 135, -7)                     -- Print card tag (white)
  love.graphics.setColor(255, 215, 0, alpha)
  for i = 1, 3, 1 do
    if card["Spec"..i]~=nil then 
      love.graphics.printf(card["Spec"..i], x, y, 270, "left", 0, sx, sy, 135, -55-(i*20))  -- Print special abilities (yellow)
    end
  end
  love.graphics.setColor(255, 255, 255, alpha)
  j = 0
  for i = 1, #tEFFECTS, 1 do                                                                -- Effect icons
    if card.Effect~=nil and card.Effect[i]>0 then
      j = j + 1
      if j<=3 then
        love.graphics.draw(iICON[tEFFECTS[i].Pic], x, y, 0, sx, sy, 133, 185-(j*45))
      elseif j>3 and j<=6 then
        love.graphics.draw(iICON[tEFFECTS[i].Pic], x, y, 0, sx, sy, -95, 185-((j-3)*45))
      else end
    end
  end
  if p~=0 and mx>x-(40*sx) and mx<x+(40*sx) and my>y+(130*sy) and my<y+(210*sy) then        -- Print movement description hover
    drawshadedrectangle(mx-160, my+25, 320, 100, 180, 180, 180)
    drawshadowtext(tPATTERN[p].Desc, mx-160, my+25, 320, "center", fHEADING)
  elseif mx>x-(144*sx) and mx<x-(76*sx) and my>y-(204*sx) and my<y-(136*sx) then            -- Print tier description hover
    drawshadedrectangle(mx-160, my+25, 320, 40, 180, 180, 180)
    drawshadowtext("May be played from turn "..card.Tier, mx-160, my+25, 320, "center", fHEADING)
  elseif mx>x+(78*sx) and mx<x+(146*sx) and my>y+(132*sy) and my<y+(206*sy) then            -- Print attack description hover
    drawshadedrectangle(mx-160, my+25, 320, 40, 180, 180, 180)
    drawshadowtext("Attacks with "..a.." damage", mx-160, my+25, 320, "center", fHEADING)
  elseif mx>x-(146*sx) and mx<x-(78*sx) and my>y+(146*sy) and my<y+(206*sy) then            -- Print health description hover
    drawshadedrectangle(mx-160, my+25, 320, 40, 180, 180, 180)
    drawshadowtext("Has "..h.." health", mx-160, my+25, 320, "center", fHEADING)
  elseif mx>x-(135*sx) and mx<x+(135*sx) and my>y+(76*sy) and my<y+(136*sy) then
    for i = 1, 3, 1 do
      if card["Spec"..i]~=nil and my>y+((56+(i*20))*sy) and my<y+((76+(i*20))*sy) then 
        j = getabilitybyname(card["Spec"..i])
        drawshadedrectangle(mx-160, my+25, 320, 70, 180, 180, 180)
        drawshadowtext(tABILITIES[j].Desc, mx-160, my+25, 320, "center", fDEFAULT)
      end
    end
  else end
  j = 0
  for i = 1, #tEFFECTS, 1 do                                                                -- Effect descriptions
    if card.Effect~=nil and card.Effect[i]>0 then
      j = j + 1
      if j<=3 and mx>x-(133*sx) and mx<x-(93*sx) and my>y-((185-(j*45))*sx) and my<y-((145-(j*45))*sx) then
        drawshadedrectangle(mx-160, my+25, 320, 70, 180, 180, 180)
        drawshadowtext("("..card.Effect[i]..") "..tEFFECTS[i].Name..": "..tEFFECTS[i].Result, mx-160, my+25, 320, "center", fDEFAULT)
      elseif j>3 and j<=6 and mx>x+(95*sx) and mx<x+(135*sx) and my>y-((185-((j-3)*45))*sx) and my<y-((145-((j-3)*45))*sx) then
        drawshadedrectangle(mx-160, my+25, 320, 70, 180, 180, 180)
        drawshadowtext("("..card.Effect[i]..") "..tEFFECTS[i].Name..": "..tEFFECTS[i].Result, mx-160, my+25, 320, "center", fDEFAULT)
      else end
    end
  end
end

function drawgame()  -- Draw the main game
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[gBACKGROUND], 0, 0)
  drawgrid()
  drawhighlighted()
  drawzones()
  drawgui()
  if gDISPLAYHAND == true then drawhand() end
  drawgrave()
  drawcombat()
  drawguihighlight()
  drawanimatedtext()
  if gZOOMED==true and gDRAGGING==false and gGAMEMODE=="game" then drawzoomed()
  elseif gDRAGGING==true and gMOUSEGRAB>tSETTINGS.FPS*0.3 and gGAMEMODE=="game" then drawgrabbed()
  else end
end

function drawzoomed()  -- Draw the zoomed in card
  local r, g, b = getcolours(tCLICK.Player)
  drawshadedrectangle(0, 0, 800, 600, 0, 0, 0)
  if tCLICK.Source=="Deck" and gTURN==tCLICK.Player and tCONTROL[gTURN]=="Player" then
    if tCLICK.Card.Available==true then
      if gethasfreeendspace(gTURN) then
        drawshadedrectangle(250, 520, 300, 60, 255, 255, 255)
        drawshadowtext("Play to Battle", 255, 525, 290, "center", fTITLE, 255, 255, 255, 255, 255, 215, 0, 255)
      else
        drawshadedrectangle(250, 520, 300, 60, 100, 100, 100)
        drawshadowtext("No available space", 255, 525, 290, "center", fTITLE)
      end
    else
      drawshadedrectangle(250, 520, 300, 60, 100, 100, 100)
      drawshadowtext("Play from Turn "..tCLICK.Card.Tier, 255, 525, 290, "center", fTITLE)
    end
  elseif tCLICK.Source=="Combat" and gTURN==tCLICK.Player and tCONTROL[gTURN]=="Player"then
    if gethaseffect(tCLICK.Card, "Immobile") or gethaseffect(tCLICK.Card, "Limp") then
      drawshadedrectangle(250, 520, 300, 60, 100, 100, 100)
      drawshadowtext("Cannot Move", 255, 525, 290, "center", fTITLE)
    else
      drawshadedrectangle(250, 520, 300, 60, 255, 255, 255)
      drawshadowtext("Move", 255, 525, 290, "center", fTITLE, 255, 255, 255, 255, 255, 215, 0, 255)
    end
  else end  
  drawcard(tCLICK.Card, 400, 300, 300, 420, r, g, b)
end

function drawgrabbed()   -- Draw the grabbed card icon
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  drawcardicon(tCLICK.Card, mx, my, 80, 80, 255, 255, 255, 100)
end

function drawgrid()  -- Draw the board/grid
  local sq_size = tSETTINGS.Boardsize/tGAME.Gridsize
  local i, j, x, y
  for i = 1, tGAME.Gridsize, 1 do
    for j = 1, tGAME.Gridsize, 1 do
      if (i+j)%2==0 then love.graphics.setColor(255, 255, 255, 200)
      else love.graphics.setColor(180, 180, 180, 200) end
      x, y = gridtoscreen(i, j)
      love.graphics.rectangle("fill", x-(sq_size/2), y-(sq_size/2), sq_size, sq_size)
    end
  end
end

function drawzones()  -- Draw the scoring zones
  local sq_size = tSETTINGS.Boardsize/tGAME.Gridsize
  local j = (tGAME.Gridsize+1)/2
  local i, x, y
  for i = 1, tGAME.Gridsize, 1 do
    x, y = gridtoscreen(i, j)
    if tZONE[i]==0 then love.graphics.setColor(255, 215, 0, 100)
    elseif tZONE[i]==1 then love.graphics.setColor(tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255)
    else love.graphics.setColor(tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255) end
    love.graphics.draw(iICON[1], x, y, 0, sq_size/iICON[1]:getWidth(), sq_size/iICON[1]:getHeight(), iICON[1]:getWidth()/2, iICON[1]:getHeight()/2)
    x, y = gridtoscreen(i, 1)
    love.graphics.setColor(tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 100)
    love.graphics.draw(iICON[3], x, y, 0, sq_size/iICON[3]:getWidth(), sq_size/iICON[3]:getHeight(), iICON[3]:getWidth()/2, iICON[3]:getHeight()/2)
    x, y = gridtoscreen(i, tGAME.Gridsize)
    love.graphics.setColor(tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 100)
    love.graphics.draw(iICON[2], x, y, 0, sq_size/iICON[2]:getWidth(), sq_size/iICON[2]:getHeight(), iICON[2]:getWidth()/2, iICON[2]:getHeight()/2)
  end
end

function drawgui()  -- Draw player names, turn indicator, hand background etc.
  local pl1, pl2 = 0, 0
  local i
  for i = 1, tGAME.Gridsize, 1 do
    if tZONE[i]==1 then pl1 = pl1 + 1
    elseif tZONE[i]==2 then pl2 = pl2 + 1
    else end
  end
  drawshadowtext(tSETTINGS.Player1, xBOARD-(tSETTINGS.Boardsize/2), yBOARD+(tSETTINGS.Boardsize/2)+10, tSETTINGS.Boardsize, "center", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B)
  drawshadowtext(tGAME.Opponent, xBOARD-(tSETTINGS.Boardsize/2), yBOARD-(tSETTINGS.Boardsize/2)-40, tSETTINGS.Boardsize, "center", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B)
  love.graphics.setColor(255, 255, 255, 255)
  if gTURN == 1 then 
    if gDAYNIGHT>=6 then
      love.graphics.draw(iICON[20], 20, 528)
      drawshadowtext(11-gDAYNIGHT, 20, 528, 32, "center", fHEADING)
    else
      love.graphics.draw(iICON[19], 20, 528)
      drawshadowtext(6-gDAYNIGHT, 20, 528, 32, "center", fHEADING)
    end
    if gROUND<=tGAME.Rounds then drawshadowtext("Turn "..math.ceil(gROUND/2).."/"..(tGAME.Rounds/2), 20, 560, 180, "left", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255) end
    if math.ceil(gROUND/2)>=tGAME.Rounds/2 then drawshadowtext("FINAL TURN", xBOARD-(tSETTINGS.Boardsize/2), yBOARD+(tSETTINGS.Boardsize/2)+40, tSETTINGS.Boardsize, "center", fHEADING)
    else drawshadowtext("MY TURN", xBOARD-(tSETTINGS.Boardsize/2), yBOARD+(tSETTINGS.Boardsize/2)+40, tSETTINGS.Boardsize, "center", fHEADING) end
    if tCONTROL[gTURN]=="Player" then
      if gDISPLAYHAND==true then drawshadowtext("End Turn", 300, 528, 190, "right", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255, 255, 255, 255, 255)
      else drawshadowtext("End Turn", 590, 528, 190, "right", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255, 255, 255, 255, 255) end
    else
      if gDISPLAYHAND==true then drawshadowtext("Thinking...", 300, 528, 190, "right", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255)
      else drawshadowtext("Thinking...", 590, 528, 190, "right", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255) end
    end
  else 
    if gDAYNIGHT>=6 then
      love.graphics.draw(iICON[20], 20, 30)
      drawshadowtext(11-gDAYNIGHT, 20, 30, 32, "center", fHEADING)
    else
      love.graphics.draw(iICON[19], 20, 30)
      drawshadowtext(6-gDAYNIGHT, 20, 30, 32, "center", fHEADING)
    end
    if gROUND<=tGAME.Rounds then drawshadowtext("Turn "..math.ceil(gROUND/2).."/"..(tGAME.Rounds/2), 20, 0, 180, "left", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255) end
    if math.ceil(gROUND/2)>=tGAME.Rounds/2 then drawshadowtext("FINAL TURN", xBOARD-(tSETTINGS.Boardsize/2), yBOARD-(tSETTINGS.Boardsize/2)-70, tSETTINGS.Boardsize, "center", fHEADING)
    else drawshadowtext("MY TURN", xBOARD-(tSETTINGS.Boardsize/2), yBOARD-(tSETTINGS.Boardsize/2)-70, tSETTINGS.Boardsize, "center", fHEADING) end
    if tCONTROL[gTURN]=="Player" then
      if gDISPLAYHAND==true then drawshadowtext("End Turn", 300, 30, 190, "right", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255, 255, 255, 255, 255)
      else drawshadowtext("End Turn", 590, 30, 190, "right", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255, 255, 255, 255, 255) end
    else
      if gDISPLAYHAND==true then drawshadowtext("Thinking...", 300, 30, 190, "right", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255)
      else drawshadowtext("Thinking...", 590, 30, 190, "right", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255) end
    end
  end
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iICON[6], 0, 0)
  if gDISPLAYHAND == true then
    drawshadedrectangle(500, 0, 300, 600, 0, 0, 0)
    love.graphics.setColor(255, 255, 255, 255)
    love.graphics.draw(iICON[6], 500, 0)
    drawshadowtext(tSETTINGS.Player1, 500, 300, 300, "center", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255)
    drawshadowtext(tGAME.Opponent, 500, 0, 300, "center", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255)
    drawshadowtext("Score: "..tGAME.Score[1].." (+"..pl1..")", 300, 560, 190, "right", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255)
    drawshadowtext("Score: "..tGAME.Score[2].." (+"..pl2..")", 300, 0, 190, "right", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255)
  else 
    love.graphics.draw(iICON[7], 768, 0)
    drawshadowtext("Score: "..tGAME.Score[1].." (+"..pl1..")", 590, 560, 190, "right", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255)
    drawshadowtext("Score: "..tGAME.Score[2].." (+"..pl2..")", 590, 0, 190, "right", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255)
  end
end

function drawguihighlight()   -- Draw overlay text from gui
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  if gTURN==1 then
    if mx>20 and mx<52 and my>528 and my<550 and gZOOMED==false and gGAMEMODE=="game" then
      drawshadedrectangle(mx, my-40, 180, 30)
      if gDAYNIGHT>=6 then drawshadowtext(11-gDAYNIGHT.." turn(s) of night remain", mx, my-40, 180, "center", fDEFAULT)
      else drawshadowtext(6-gDAYNIGHT.." turn(s) of day remain", mx, my-40, 180, "center", fDEFAULT) end
    end
  else
    if mx>20 and mx<52 and my>30 and my<62 and gZOOMED==false and gGAMEMODE=="game" then
      drawshadedrectangle(mx, my+25, 180, 30)
      if gDAYNIGHT>=6 then drawshadowtext(11-gDAYNIGHT.." turn(s) of night remain", mx, my+25, 180, "center", fDEFAULT)
      else drawshadowtext(6-gDAYNIGHT.." turn(s) of day remain", mx, my+25, 180, "center", fDEFAULT) end
    end
  end
end

function drawhighlighted()  -- Draw highlighted grid squares
  local sq_size = tSETTINGS.Boardsize/tGAME.Gridsize
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local r, g, b = getcolours(gTURN)
  local i, j, x, y
  for i = 1, tGAME.Gridsize, 1 do
    for j = 1, tGAME.Gridsize, 1 do
      x, y = gridtoscreen(i, j)
      if tGRID[j][i]==true then drawthickrectangle(x-(sq_size/2), y-(sq_size/2), sq_size, sq_size, r, g, b) end
    end
  end
end

function drawhand()  -- Draw player's hand in panel (if enabled)
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local rows, cols = getdimensions(tGAME.Pieces)
  local icon_size = math.min(260/rows, 260/cols)
  local r, g, b
  local i, pl
  for pl = 1, 2, 1 do  -- Draw non-animated cards first
    if next(tDECK[pl])~=nil then
      r, g, b = getcolours(pl)
      for i = 1, #tDECK[pl], 1 do
        if tDECK[pl][i].Animate == false then
          if tDECK[pl][i].Available==true then drawcardicon(tDECK[pl][i], tDECK[pl][i].X, tDECK[pl][i].Y, icon_size, icon_size, 255, 255, 255, 255)
          else drawcardicon(tDECK[pl][i], tDECK[pl][i].X, tDECK[pl][i].Y, icon_size, icon_size, 100, 100, 100, 255) end
          if next(tCLICK)~=nil and tCLICK.Player==pl and tCONTROL[pl]=="Player" and tCLICK.Source=="Deck" and tCLICK.Index==i then drawthickrectangle(tDECK[pl][i].X-(icon_size/2), tDECK[pl][i].Y-(icon_size/2), icon_size, icon_size, r, g, b) end
        end
      end
    end
  end
  for pl = 1, 2, 1 do  -- Then draw animated cards on top
    if next(tDECK[pl])~=nil then
      r, g, b = getcolours(pl)
      for i = 1, #tDECK[pl], 1 do
        if tDECK[pl][i].Animate == true then
          if tDECK[pl][i].Available==true then drawcardicon(tDECK[pl][i], tDECK[pl][i].X, tDECK[pl][i].Y, icon_size, icon_size, 255, 255, 255, 255)
          else drawcardicon(tDECK[pl][i], tDECK[pl][i].X, tDECK[pl][i].Y, icon_size, icon_size, 100, 100, 100, 255) end
          if next(tCLICK)~=nil and tCLICK.Player==pl and tCLICK.Source=="Deck" and tCLICK.Index==i then drawthickrectangle(tDECK[pl][i].X-(icon_size/2), tDECK[pl][i].Y-(icon_size/2), icon_size, icon_size, r, g, b) end
        end
      end
    end
  end
  for pl = 1, 2, 1 do  -- Then draw label rectangle by mouse
    if next(tDECK[pl])~=nil and gZOOMED == false and gGAMEMODE=="game" then
      for i = 1, #tDECK[pl], 1 do
        if mx>tDECK[pl][i].X-(icon_size/2) and mx<tDECK[pl][i].X+(icon_size/2) and my>tDECK[pl][i].Y-(icon_size/2) and my<tDECK[pl][i].Y+(icon_size/2) then
          drawshadedrectangle(mx-80, my+30, 160, 25, 100, 100, 100)
          drawshadowtext(tDECK[pl][i].Name, mx-80, my+30, 160, "center", fDEFAULT)
        end
      end
    end
  end
end

function drawgrave()   -- Draw cards on the way to grave!
  local sq_size = (tSETTINGS.Boardsize/tGAME.Gridsize)*0.8
  local r, g, b
  local i, pl
  for pl = 1, 2, 1 do
    if next(tGRAVE[pl])~=nil then
      r, g, b = getcolours(pl)
      for i = 1, #tGRAVE[pl], 1 do
        drawcardicon(tGRAVE[pl][i], tGRAVE[pl][i].X, tGRAVE[pl][i].Y, sq_size, sq_size, 100, 100, 100, 255)
        love.graphics.setColor(r, g, b, 255)
        love.graphics.rectangle("line", tGRAVE[pl][i].X-(sq_size/2)-2, tGRAVE[pl][i].Y-(sq_size/2)-2, sq_size+4, sq_size+4)
      end
    end
  end
end

function drawcombat()  -- Draw combat cards
  local mx, my = love.mouse.getX()/gRATIOX, love.mouse.getY()/gRATIOY
  local sq_size = (tSETTINGS.Boardsize/tGAME.Gridsize)*0.8
  local r, g, b
  local i, pl
  for pl = 1, 2, 1 do  -- Draw non-animated cards first
    if next(tCOMBAT[pl])~=nil then
      r, g, b = getcolours(pl)
      for i = 1, #tCOMBAT[pl], 1 do
        if tCOMBAT[pl][i].Animate == false and (not gethaseffect(tCOMBAT[pl][i], "Hidden") or (gTURN==pl and tCONTROL[pl]=="Player")) then
          if gethaseffect(tCOMBAT[pl][i], "Immobile") or gethaseffect(tCOMBAT[pl][i], "Limp") then drawcardicon(tCOMBAT[pl][i], tCOMBAT[pl][i].X, tCOMBAT[pl][i].Y, sq_size, sq_size, 100, 100, 100, 255)
          else drawcardicon(tCOMBAT[pl][i], tCOMBAT[pl][i].X, tCOMBAT[pl][i].Y, sq_size, sq_size) end
          if next(tCLICK)~=nil and tCLICK.Player==pl and tCLICK.Source=="Combat" and tCLICK.Index==i then drawthickrectangle(tCOMBAT[pl][i].X-(sq_size/2)-2, tCOMBAT[pl][i].Y-(sq_size/2)-2, sq_size+4, sq_size+4, r, g, b, gFLASH)
          else
            love.graphics.setColor(r, g, b, 255)
            love.graphics.rectangle("line", tCOMBAT[pl][i].X-(sq_size/2)-2, tCOMBAT[pl][i].Y-(sq_size/2)-2, sq_size+4, sq_size+4)
          end
        end
      end
    end
  end
  for pl = 1, 2, 1 do  -- Then draw animated cards next
    if next(tCOMBAT[pl])~=nil then
      r, g, b = getcolours(pl)
      for i = 1, #tCOMBAT[pl], 1 do
        if tCOMBAT[pl][i].Animate == true and (not gethaseffect(tCOMBAT[pl][i], "Hidden") or (gTURN==pl and tCONTROL[pl]=="Player")) then
          drawcardicon(tCOMBAT[pl][i], tCOMBAT[pl][i].X, tCOMBAT[pl][i].Y, sq_size, sq_size)
          if next(tCLICK)~=nil and tCLICK.Player==pl and tCLICK.Source=="Combat" and tCLICK.Index==i then drawthickrectangle(tCOMBAT[pl][i].X-(sq_size/2)-2, tCOMBAT[pl][i].Y-(sq_size/2)-2, sq_size+4, sq_size+4, r, g, b, gFLASH)
          else
            love.graphics.setColor(r, g, b, 255)
            love.graphics.rectangle("line", tCOMBAT[pl][i].X-(sq_size/2)-2, tCOMBAT[pl][i].Y-(sq_size/2)-2, sq_size+4, sq_size+4)
          end
        end
      end
    end
  end
  for pl = 1, 2, 1 do   -- Then draw label rectangle by mouse
    if next(tCOMBAT[pl])~=nil and gZOOMED == false and gGAMEMODE=="game" then
      for i = 1, #tCOMBAT[pl], 1 do
        if mx>tCOMBAT[pl][i].X-(sq_size/2) and mx<tCOMBAT[pl][i].X+(sq_size/2) and my>tCOMBAT[pl][i].Y-(sq_size/2) and my<tCOMBAT[pl][i].Y+(sq_size/2) and (not gethaseffect(tCOMBAT[pl][i], "Hidden") or (gTURN==pl and tCONTROL[pl]=="Player")) then
          drawshadedrectangle(mx-80, my+30, 160, 25, 100, 100, 100)
          drawshadowtext(tCOMBAT[pl][i].Name, mx-80, my+30, 160, "center", fDEFAULT)
        end
      end
    end
  end
end

function drawanimatedtext()  -- Draw animated text
  local i, j, k
  local x, y, z, t
  for j = 1, 2, 1 do
    if next(tDECK[j])~=nil then
      for i = 1, #tDECK[j], 1 do
        if next(tDECK[j][i].FloatyText)~=nil then
          for k = 1, #tDECK[j][i].FloatyText, 1 do
            x = tDECK[j][i].FloatyText[k].X
            y = tDECK[j][i].FloatyText[k].Y-tDECK[j][i].FloatyText[k].Timer
            z = math.floor(255*((tSETTINGS.FloatTime-tDECK[j][i].FloatyText[k].Timer)/tSETTINGS.FloatTime))
            t = tDECK[j][i].FloatyText[k].Text
            if string.find(t, "+") then drawshadowtext(t, x-100, y, 200, "center", fHEADING, 0, 255, 0, z)
            elseif string.find(t, "-") then drawshadowtext(t, x-100, y, 200, "center", fHEADING, 255, 0, 0, z)
            else drawshadowtext(t, x-100, y, 200, "center", fHEADING, 255, 255, 255, z) end
          end
        end
      end
    end
    if next(tCOMBAT[j])~=nil then
      for i = 1, #tCOMBAT[j], 1 do
        if next(tCOMBAT[j][i].FloatyText)~=nil then
          for k = 1, #tCOMBAT[j][i].FloatyText, 1 do
            x = tCOMBAT[j][i].FloatyText[k].X
            y = tCOMBAT[j][i].FloatyText[k].Y-tCOMBAT[j][i].FloatyText[k].Timer
            z = math.floor(255*((tSETTINGS.FloatTime-tCOMBAT[j][i].FloatyText[k].Timer)/tSETTINGS.FloatTime))
            t = tCOMBAT[j][i].FloatyText[k].Text
            if string.find(t, "+") then drawshadowtext(t, x-100, y, 200, "center", fHEADING, 0, 255, 0, z)
            elseif string.find(t, "-") then drawshadowtext(t, x-100, y, 200, "center", fHEADING, 255, 0, 0, z)
            else drawshadowtext(t, x-100, y, 200, "center", fHEADING, 255, 255, 255, z) end
          end
        end
      end
    end
    if next(tGRAVE[j])~=nil then
      for i = 1, #tGRAVE[j], 1 do
        if next(tGRAVE[j][i].FloatyText)~=nil then
          for k = 1, #tGRAVE[j][i].FloatyText, 1 do
            x = tGRAVE[j][i].FloatyText[k].X
            y = tGRAVE[j][i].FloatyText[k].Y-tGRAVE[j][i].FloatyText[k].Timer
            z = math.floor(255*((tSETTINGS.FloatTime-tGRAVE[j][i].FloatyText[k].Timer)/tSETTINGS.FloatTime))
            t = tGRAVE[j][i].FloatyText[k].Text
            if string.find(t, "+") then drawshadowtext(t, x-100, y, 200, "center", fHEADING, 0, 255, 0, z)
            elseif string.find(t, "-") then drawshadowtext(t, x-100, y, 200, "center", fHEADING, 255, 0, 0, z)
            else drawshadowtext(t, x-100, y, 200, "center", fHEADING, 255, 255, 255, z) end
          end
        end
      end
    end
  end
end

function drawscore()  -- Draw the final score
  drawgame()
  drawshadedrectangle(0, 0, 800, 600, 0, 0, 0)
  drawshadowtext("Final Score", 0, 50, 800, "center", fTITLE)
  drawshadowtext(tSETTINGS.Player1, 0, 150, 300, "center", fTITLE, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B)
  drawshadowtext(tGAME.Opponent, 500, 150, 300, "center", fTITLE, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B)
  drawshadowtext(tGAME.Score[1], 0, 230, 300, "center", fHUGE, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B)
  drawshadowtext(tGAME.Score[2], 500, 230, 300, "center", fHUGE, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B)
  if gNEWLEVEL~=0 then drawshadowtext("You have unlocked a new level!", 0, 500, 800, "center", fTITLE) end
  if gNEWCARD~=0 then 
    drawshadowtext("You have unlocked a new card!", 0, 450, 800, "center", fTITLE)
    drawcard(tBLUEPRINT[gNEWCARD], 400, 280, 200, 280, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B) 
  end
  if tGAME.Winner==1 and gNEWGOLD~=0 then drawshadowtext("You gained "..gNEWGOLD.." gold!", 0, 550, 800, "center", fTITLE)
  elseif tGAME.Winner==1 then drawshadowtext(tSETTINGS.Player1.." won the game!", 0, 550, 800, "center", fTITLE)
  elseif tGAME.Winner==2 then drawshadowtext(tGAME.Opponent.." won the game!", 0, 550, 800, "center", fTITLE)
  else drawshadowtext("The game was a draw!", 0, 550, 800, "center", fTITLE) end
end

function drawnewturn()  -- Draw the new turn screen
  if gTURN==1 then drawshadowtext(tSETTINGS.Player1..": Your Turn", 0, 280, 800, "center", fTITLE, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B)
  else drawshadowtext(tGAME.Opponent..": Your Turn", 0, 280, 800, "center", fTITLE, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B) end
end

function drawsettings()  -- Draw the settings screen
  local list = {"FPS", "Sound", "Music", "MusicVol", "Gridsize", "Rounds", "Pieces", "Player1", "Player2"}
  local i
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(iBACKGROUND[12], 0, 0)
  drawshadedrectangle(0, 0, 800, 600, 100, 100, 100)
  drawshadowtext("Settings", 0, 25, 800, "center", fTITLE)
  for i = 1, #list, 1 do
    drawshadowtext(list[i], 0, (i*40)+40, 390, "right", fHEADING)
    if list[i]=="MusicVol" then drawshadowtext((tSETTINGS[list[i]]*100).."%", 410, (i*40)+40, 190, "left", fHEADING, 255, 215, 0, 255, 255, 255, 255, 255)
    elseif list[i]=="Rounds" then drawshadowtext(tSETTINGS[list[i]]/2, 410, (i*40)+40, 190, "left", fHEADING, 255, 215, 0, 255, 255, 255, 255, 255)
    elseif list[i]=="Player1" then 
      drawshadowtext(tSETTINGS[list[i]], 410, (i*40)+40, 190, "left", fHEADING, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B, 255, 255, 255, 255, 255)
      drawshadowtext(tSETTINGS.P1R, 610, (i*40)+40, 50, "center", fHEADING, 255, 0, 0, 255, 255, 255, 255, 255)
      drawshadowtext(tSETTINGS.P1G, 670, (i*40)+40, 50, "center", fHEADING, 0, 255, 0, 255, 255, 255, 255, 255)
      drawshadowtext(tSETTINGS.P1B, 730, (i*40)+40, 50, "center", fHEADING, 0, 0, 255, 255, 255, 255, 255, 255)
    elseif list[i]=="Player2" then 
      drawshadowtext(tSETTINGS[list[i]], 410, (i*40)+40, 190, "left", fHEADING, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B, 255, 255, 255, 255, 255)
      drawshadowtext(tSETTINGS.P2R, 610, (i*40)+40, 50, "center", fHEADING, 255, 0, 0, 255, 255, 255, 255, 255)
      drawshadowtext(tSETTINGS.P2G, 670, (i*40)+40, 50, "center", fHEADING, 0, 255, 0, 255, 255, 255, 255, 255)
      drawshadowtext(tSETTINGS.P2B, 730, (i*40)+40, 50, "center", fHEADING, 0, 0, 255, 255, 255, 255, 255, 255)
    else drawshadowtext(tSETTINGS[list[i]], 410, (i*40)+40, 190, "left", fHEADING, 255, 215, 0, 255, 255, 255, 255, 255) end
  end
  drawshadowtext("Save/Exit", 300, 535, 200, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255)
end

function drawplayername()  -- Draw the player name screen
  local row1 = {"Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P", "DEL"}
  local row2 = {"CAPS", "A", "S", "D", "F", "G", "H", "J", "K", "L", "ENTER"}
  local row3 = {"Z", "X", "C", "V", "B", "N", "M"}
  local i
  drawshadowtext("Player Name", 0, 25, 800, "center", fTITLE)
  if gSELECT==1 then drawshadowtext(tSETTINGS.Player1, 0, 75, 800, "center", fTITLE, tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B)
  else drawshadowtext(tSETTINGS.Player2, 0, 75, 800, "center", fTITLE, tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B) end
  for i = 1, #row1, 1 do
    if row1[i]=="DEL" then drawshadowtext("DEL", 700, 200, 75, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255)
    elseif gSUBSELECT==0 then drawshadowtext(string.lower(row1[i]), (i*50)+75, 200, 50, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255)
    else drawshadowtext(row1[i], (i*50)+75, 200, 50, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255) end
  end
  for i = 1, #row2, 1 do
    if row2[i]=="CAPS" then drawshadowtext("CAPS", 25, 300, 75, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255)
    elseif row2[i]=="ENTER" then drawshadowtext("ENTER", 675, 300, 100, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255)
    elseif gSUBSELECT==0 then drawshadowtext(string.lower(row2[i]), (i*50)+50, 300, 50, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255)
    else drawshadowtext(row2[i], (i*50)+50, 300, 50, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255) end
  end
  for i = 1, #row3, 1 do
    if gSUBSELECT==0 then drawshadowtext(string.lower(row3[i]), (i*50)+125, 400, 50, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255)
    else drawshadowtext(row3[i], (i*50)+125, 400, 50, "center", fTITLE, 255, 215, 0, 255, 255, 255, 255, 255) end
  end
end

function love.mousepressed(x, y, button, istouch, presses)  -- Record any click, based on gamemode
  x,y = x/gRATIOX, y/gRATIOY
  if _G["click"..gGAMEMODE]~=nil then
    playsound(1)
    _G["click"..gGAMEMODE](x,y)
  end
end

function clickcard(card, source, player, index)  -- A card has been clicked
  tCLICK = {}
  tCLICK.Card=card
  tCLICK.Source=source
  tCLICK.Player=player
  tCLICK.Index=index
  gDRAGGING = true
  gZOOMED = true
end

function clickmenu(x,y)  -- Click menu
  if x>300 and x<500 and y>200 and y<228 then 
    tCONTROL = {"Player", "AI"}
    tDECK = {}
    gSELECT = tPLAYER.LEVEL
    gSUBSELECT = 0
    gGAMEMODE = "map"
  elseif x>300 and x<500 and y>250 and y<278 then
    tCONTROL = {"Player", "Player"}
    tDECK = {}
    tDECK[1], tDECK[2] = {}, {}
    readcollection(tCOLLECTION)
    tGAME = {Gridsize=tSETTINGS.Gridsize, Rounds=tSETTINGS.Rounds, Pieces=tSETTINGS.Pieces, Opponent=tSETTINGS.Player2}
    gLEVEL, gDIFFICULTY, gTURN = 0, 0, 1
    gSELECT, gSUBSELECT, gPAGE, gZOOMED = 0, 0, 1, false
    tCLICK = {}
    gGAMEMODE = "selectdeck"
  elseif x>300 and x<500 and y>300 and y<328 then
    readcollection(tBLUEPRINT)
    gSELECT, gSUBSELECT, gPAGE, gZOOMED = 0, 0, 1, false
    tCLICK = {}
    gGAMEMODE = "gallery"
  elseif x>300 and x<500 and y>350 and y<378 then gGAMEMODE = "settings"
  elseif x>300 and x<500 and y>400 and y<428 then quitgame()
  else end
end

function clickmap(x,y)  -- Click campaign map
  local click = false
  local i
  if gSUBSELECT~=0 then
    if x>=0 and x<=266 and y>=100 and y<=500 then gSUBSELECT = 1
    elseif x>=267 and x<=533 and y>=100 and y<=500 then gSUBSELECT = 2
    elseif x>=534 and x<=800 and y>=100 and y<=500 then gSUBSELECT = 3
    else gSUBSELECT=0 end
    if gSUBSELECT~=0 then
      gLEVEL, gDIFFICULTY, gTURN = gSELECT, gSUBSELECT, 1
      tGAME = readsingletable("ai/game"..tostring((3*(gLEVEL-1))+gDIFFICULTY)..".txt")
      tDECK[1], tDECK[2] = {}, loaddeck("ai/deck"..tostring((3*(gLEVEL-1))+gDIFFICULTY)..".txt")
      readcollection(tCOLLECTION)
      gSELECT, gSUBSELECT, gPAGE, gZOOMED = 0, 0, 1, false
      tCLICK = {}
      gGAMEMODE = "selectdeck"
    end
  elseif x>720 and x<790 and y>560 and y<588 then startmenu()
  else
    for i = 1, tPLAYER.LEVEL, 1 do
      if distancebetween(x, y, tMAP[i].X, tMAP[i].Y)<=80 then
        click = true
        gSELECT = i
        gSUBSELECT = 4
      end
    end
    if click == false then gSELECT, gSUBSELECT = 0, 0 end
  end
end

function clickselectdeck(x,y)  -- Click select deck
  local list = {}
  local a, b, i, clicked
  if gSELECT~=0 and gSUBSELECT~=0 then list = getlistofcards(tCOLLECTION, tCLASS[gSUBSELECT], tTIER[gSELECT], "Alphabetical")
  elseif gSELECT~=0 then list = getlistofcards(tCOLLECTION, "All", tTIER[gSELECT], "Alphabetical")
  elseif gSUBSELECT~=0 then list = getlistofcards(tCOLLECTION, tCLASS[gSUBSELECT], 0, "Tier")
  else list = getlistofcards(tCOLLECTION, "All", 0, "Tier") end
  if gZOOMED==true then
    clicked = false
    if x>250 and x<550 and y>520 and y<580 then
      if getisnameinlist(tCLICK.Card, tDECK[gTURN]) then removecardfromdeck(tCLICK.Card, gTURN)
      else
        addcardtodeck(tCLICK.Card, gTURN)
        if #tDECK[gTURN]>tGAME.Pieces then table.remove(tDECK[gTURN], 1) end
      end
      clicked = true
    elseif x>250 and x<550 and y>90 and y<510 then
      clicked = true
    else end
    if clicked == false then
      gZOOMED = false
      tCLICK = {}
    end
  elseif x>565 and x<785 and y>15 and y<55 and #tDECK[gTURN]==math.min(tGAME.Pieces,#tCOLLECTION) then
    tDECK[gTURN] = getlistofcards(tDECK[gTURN])
    if gTURN==1 and tCONTROL[2]=="Player" then gTURN = 2
    else
      newgame()
      if tCONTROL[2]=="Player" then 
        gWAIT = 0
        gGAMEMODE = "newturn"
      else
        tPLAYER.GAMES = tPLAYER.GAMES + 1
        writesingletable("player.txt", tPLAYER)
      end
    end
  elseif next(list)~=nil and gZOOMED~=true and gPAGE>1 and x>54 and x<122 and y>522 and y<590 then gPAGE = gPAGE - 1 
  elseif next(list)~=nil and gZOOMED~=true and ((gPAGE-1)*20)+20<#list and x>678 and x<746 and y>522 and y<590 then gPAGE = gPAGE + 1
  elseif next(list)~=nil and gZOOMED~=true and x>170 and x<630 then
    a, b = 220, 120
    for i = ((gPAGE-1)*20)+1, math.min(((gPAGE-1)*20)+20, #list), 1 do
      if x>a+20 and x<a+50 and y>b-50 and y<b-20 then
        clickcard(list[i], "List", gTURN, i)
      elseif x>a-50 and x<a+50 and y>b-50 and y<b+50 then
        if getisnameinlist(list[i], tDECK[gTURN]) then removecardfromdeck(list[i], gTURN)
        else
          addcardtodeck(list[i], gTURN)
          if #tDECK[gTURN]>tGAME.Pieces then table.remove(tDECK[gTURN], 1) end 
        end
      else end
      a = a + 120
      if a>580 then
        a = 220
        b = b + 105
      end
    end
  elseif next(tTIER)~=nil and gZOOMED~=true and x>30 and x<145 then
    if y>70 and y<110 then gSELECT = 0
    else
      for i = 1, #tTIER, 1 do
        if y>(i*50)+70 and y<(i*50)+110 then gSELECT = i end
      end
    end
  elseif next(tCLASS)~=nil and gZOOMED~=true and x>655 and x<770 then
    if y>70 and y<110 then gSUBSELECT = 0
    else
      for i = 1, #tCLASS, 1 do
        if y>(i*50)+70 and y<(i*50)+110 then gSUBSELECT = i end
      end
    end
  elseif x<=32 and y<=32 then 
    gFADE = 255
    startmenu()
  else end
end

function clickgallery(x,y)  -- Click gallery
  local list = {}
  local a, b, i, clicked
  if gSELECT~=0 and gSUBSELECT~=0 then list = getlistofcards(tBLUEPRINT, tCLASS[gSUBSELECT], tTIER[gSELECT], "Alphabetical")
  elseif gSELECT~=0 then list = getlistofcards(tBLUEPRINT, "All", tTIER[gSELECT], "Alphabetical")
  elseif gSUBSELECT~=0 then list = getlistofcards(tBLUEPRINT, tCLASS[gSUBSELECT], 0, "Tier")
  else list = getlistofcards(tBLUEPRINT, "All", 0, "Tier") end
  if gZOOMED==true then
    clicked = false
    if x>250 and x<550 and y>520 and y<580 then
      if getisnameinlist(tCLICK.Card, tCOLLECTION) then  -- Do nothing
      elseif getiscardunlocked(tCLICK.Card) and tPLAYER.GOLD>=tCLICK.Card.Cost then 
        tPLAYER.GOLD=tPLAYER.GOLD-tCLICK.Card.Cost
        addcardtocollection(tCLICK.Card.Name)
        -- play nice bought a card sound
        writesingletable("player.txt", tPLAYER)
        savedeck(tCOLLECTION, "collection.txt")
      else
        drawshadedrectangle(250, 520, 300, 60, 100, 100, 100)
        drawshadowtext("Not yet unlocked", 255, 525, 290, "center", fTITLE)
      end
      clicked = true
    elseif x>250 and x<550 and y>90 and y<510 then
      clicked = true
    else end
    if clicked == false then
      gZOOMED = false
      tCLICK = {}
    end
  elseif next(list)~=nil and gZOOMED~=true and gPAGE>1 and x>54 and x<122 and y>522 and y<590 then gPAGE = gPAGE - 1 
  elseif next(list)~=nil and gZOOMED~=true and ((gPAGE-1)*20)+20<#list and x>678 and x<746 and y>522 and y<590 then gPAGE = gPAGE + 1
  elseif next(list)~=nil and gZOOMED~=true and x>170 and x<630 then
    a, b = 220, 120
    for i = ((gPAGE-1)*20)+1, math.min(((gPAGE-1)*20)+20, #list), 1 do
      if x>a-50 and x<a+50 and y>b-50 and y<b+50 then
        clickcard(list[i], "List", gTURN, i)
      end
      a = a + 120
      if a>580 then
        a = 220
        b = b + 105
      end
    end
  elseif next(tTIER)~=nil and gZOOMED~=true and x>30 and x<145 then
    if y>70 and y<110 and gSELECT~=0 then 
      gSELECT = 0
      gPAGE = 1
    else
      for i = 1, #tTIER, 1 do
        if y>(i*50)+70 and y<(i*50)+110 and gSELECT~=i then 
          gSELECT = i
          gPAGE = 1
        end
      end
    end
  elseif next(tCLASS)~=nil and gZOOMED~=true and x>655 and x<770 then
    if y>20 and y<60 and gSUBSELECT~=0 then 
      gSUBSELECT = 0
      gPAGE = 1
    else
      for i = 1, #tCLASS, 1 do
        if y>(i*50)+20 and y<(i*50)+60 and gSUBSELECT~=i then 
          gSUBSELECT = i
          gPAGE = 1
        end
      end
    end
  elseif x<=32 and y<=32 then 
    gFADE = 255
    startmenu()
  else end
end

function clickgame(x,y)  -- Click main game
  local clicked
  if gZOOMED == true then    -- Zoomed in, click on card or button only
    if x>250 and x<550 and y>520 and y<580 and tCLICK.Source=="Deck" and gTURN==tCLICK.Player and tCONTROL[gTURN]=="Player" and tCLICK.Card.Available==true and gethasfreeendspace(gTURN) then
      gZOOMED = false
    elseif x>250 and x<550 and y>520 and y<580 and tCLICK.Source=="Combat" and gTURN==tCLICK.Player and tCONTROL[gTURN]=="Player" then
      gZOOMED = false
    elseif x>250 and x<550 and y>90 and y<510 then  -- Clicked on the card, no change

    else 
      gZOOMED = false
      gACTION=""
      tCLICK={}
      highlightgrid()
    end
  elseif x<=32 and y<=32 then   -- Clicked to quit
    startmenu()
  elseif x>=768 and x<=800 and y>=0 and y<=32 and gDISPLAYHAND==false then    -- Show hand panel
    gDISPLAYHAND=true
    xBOARD = 250
    arrangecombat()
  elseif x>=500 and x<=532 and y>=0 and y<=32 and gDISPLAYHAND==true then     -- Hide hand panel
    gDISPLAYHAND=false
    xBOARD = 400
    arrangecombat()
  elseif x>=xBOARD-(tSETTINGS.Boardsize/2) and x<=xBOARD+(tSETTINGS.Boardsize/2) and y>=yBOARD-(tSETTINGS.Boardsize/2) and y<=yBOARD+(tSETTINGS.Boardsize/2) then clickboard(x, y)      -- Clicked the board
  elseif x>=500 and x<=800 and y>=0 and y<=600 and gDISPLAYHAND==true then clickhand(x, y)    -- Clicked on a hand
  elseif gTURN==1 and tCONTROL[1]=="Player" and gDISPLAYHAND==true and x>300 and x<490 and y>528 and y<556 then gWAIT=tSETTINGS.FPS  -- Clicked skip turn
  elseif gTURN==1 and tCONTROL[1]=="Player" and gDISPLAYHAND==false and x>590 and x<780 and y>528 and y<556 then gWAIT=tSETTINGS.FPS -- Clicked skip turn
  elseif gTURN==2 and tCONTROL[2]=="Player" and gDISPLAYHAND==true and x>300 and x<490 and y>30 and y<58 then gWAIT=tSETTINGS.FPS    -- Clicked skip turn
  elseif gTURN==2 and tCONTROL[2]=="Player" and gDISPLAYHAND==false and x>590 and x<780 and y>30 and y<58 then gWAIT=tSETTINGS.FPS   -- Clicked skip turn
  else end
end

function clickboard(x,y)  -- Clicked on the board
  local sq_size = tSETTINGS.Boardsize/tGAME.Gridsize
  local gx, gy = screentogrid(x, y)
  local pl, index = getsquare(gx, gy)
  local i, j, diffx, diffy
  if gACTION=="Play" and tGRID[gy][gx]==true then                     -- Played a card to a free space
    playcard(tCLICK.Player, tCLICK.Index, gx, gy)
  elseif gACTION=="Move" and tGRID[gy][gx]==true then                 -- Move a card to an available space or attack another card
    if pl==0 and index==0 then
      movecard(tCLICK.Player, tCLICK.Index, gx, gy)
    elseif pl~=gTURN then
      if gethasability(tCLICK.Card, "Swipe") then       -- Swipe three spaces together
        if tCLICK.Card.GX==tCOMBAT[pl][index].GX then      -- On same column
          if tCOMBAT[pl][index].GX-1>=1 then 
            i, j = getsquare(tCOMBAT[pl][index].GX-1, tCOMBAT[pl][index].GY)
            if i~=0 and i~=gTURN then
              addfloatytext(tCOMBAT[i][j], "-Swipe-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Swipe")
            end
          end
          pl, index = getsquare(gx, gy)          
          if tCOMBAT[pl][index].GX+1<=tGAME.Gridsize then 
            i, j = getsquare(tCOMBAT[pl][index].GX+1, tCOMBAT[pl][index].GY)
            if i~=0 and i~=gTURN then 
              addfloatytext(tCOMBAT[i][j], "-Swipe-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Swipe")
            end
          end
          pl, index = getsquare(gx, gy)
        elseif tCLICK.Card.GY==tCOMBAT[pl][index].GY then  -- On same row
          if tCOMBAT[pl][index].GY-1>=1 then 
            i, j = getsquare(tCOMBAT[pl][index].GX, tCOMBAT[pl][index].GY-1)
            if i~=0 and i~=gTURN then
              addfloatytext(tCOMBAT[i][j], "-Swipe-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Swipe")
            end
          end
          pl, index = getsquare(gx, gy)
          if tCOMBAT[pl][index].GY+1<=tGAME.Gridsize then 
            i, j = getsquare(tCOMBAT[pl][index].GX, tCOMBAT[pl][index].GY+1)
            if i~=0 and i~=gTURN then 
              addfloatytext(tCOMBAT[i][j], "-Swipe-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Swipe")
            end
          end
          pl, index = getsquare(gx, gy)
        else end
      end
      if gethasability(tCLICK.Card, "Tail Whip") then    -- Attack two opposite spaces together
        diffx = tCOMBAT[pl][index].GX-tCLICK.Card.GX
        diffy = tCOMBAT[pl][index].GY-tCLICK.Card.GY
        if diffx~=0 or diffy~=0 then
          i, j = tCLICK.Card.GX-diffx, tCLICK.Card.GY-diffy
          if i>=1 and i<=tGAME.Gridsize and j>=1 and j<=tGAME.Gridsize then
            i, j = getsquare(i, j)
            if i~=0 and i~=gTURN then
              addfloatytext(tCOMBAT[i][j], "-Tail Whip-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Tail Whip")
            end
          end
        end
      end
      attackcard(tCLICK.Player, tCLICK.Index, pl, index)
    else end
  else
    if pl~=0 and index~=0 then
      clickcard(tCOMBAT[pl][index], "Combat", pl, index)
      if pl==gTURN and tCONTROL[pl]=="Player" and not gethaseffect(tCOMBAT[pl][index], "Immobile") and not gethaseffect(tCOMBAT[pl][index], "Limp") then
        gACTION = "Move"
        highlightgrid()
      end
    else
      gACTION = ""
      tCLICK = {}
      highlightgrid()
    end
  end
end

function clickhand(x,y)  -- Clicked on a hand
  local rows, cols = getdimensions(tGAME.Pieces)
  local icon_size = math.min(260/rows, 260/cols)
  local i, j
  for j = 1, 2, 1 do
    if next(tDECK[j])~=nil then
      for i = 1, #tDECK[j], 1 do
        if x>tDECK[j][i].X-(icon_size*0.5) and x<tDECK[j][i].X+(icon_size*0.5) and y>tDECK[j][i].Y-(icon_size*0.5) and y<tDECK[j][i].Y+(icon_size*0.5) then
          tCLICK, gACTION = {}, ""
          highlightgrid()
          clickcard(tDECK[j][i], "Deck", j, i)
          if tDECK[j][i].Available==true then
            gACTION = "Play"
            highlightgrid()
          end
        end
      end
    end
  end
end

function clickscore(x,y)  -- Clicked on score page
  startmenu()
end

function clicknewturn(x,y)  -- Clicked new turn page
  gGAMEMODE = "game"
end

function clicksettings(x,y)  -- Clicked a setting?
  local list = {"FPS", "Sound", "Music", "MusicVol", "Gridsize", "Rounds", "Pieces", "Player1", "Player2"}
  local i
  for i = 1, #list, 1 do
    if x>410 and x<600 and y>(i*40)+40 and y<(i*40)+68 then
      if list[i]=="FPS" then tSETTINGS.FPS = getnextoption(tSETTINGS.FPS, {50, 60, 70, 80, 90, 100})
      elseif list[i]=="Sound" then 
        tSETTINGS.Sound = getnextoption(tSETTINGS.Sound, {"Yes", "No"})
        if tSETTINGS.Sound=="Yes" then playsound(1) end
      elseif list[i]=="Music" then 
        tSETTINGS.Music = getnextoption(tSETTINGS.Music, {"Yes", "No"})
        if tSETTINGS.Music=="Yes" then musicplay(1)
        else musicstop() end
      elseif list[i]=="MusicVol" then
        tSETTINGS.MusicVol = getnextoption(tSETTINGS.MusicVol, {0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5, 0.55, 0.6, 0.65, 0.7, 0.75, 0.8, 0.85, 0.9, 0.95, 1})
        musicsetvolume(tSETTINGS.MusicVol)
      elseif list[i]=="Gridsize" then tSETTINGS.Gridsize = getnextoption(tSETTINGS.Gridsize, {3, 5, 7, 9})
      elseif list[i]=="Rounds" then tSETTINGS.Rounds = getnextoption(tSETTINGS.Rounds, {18, 50, 98, 162})
      elseif list[i]=="Pieces" then tSETTINGS.Pieces = getnextoption(tSETTINGS.Pieces, {1, 2, 3, 4, 5, 6, 7, 8, 9})
      elseif list[i]=="Player1" then gSELECT, gSUBSELECT, gGAMEMODE = 1, 0, "playername"
      elseif list[i]=="Player2" then gSELECT, gSUBSELECT, gGAMEMODE = 2, 0, "playername"
      else end
    elseif x>610 and x<660 and y>(i*40)+40 and y<(i*40)+68 then
      if list[i]=="Player1" then 
        tSETTINGS.P1R = tSETTINGS.P1R + 5
        if tSETTINGS.P1R>255 then tSETTINGS.P1R = 0 end
      elseif list[i]=="Player2" then
        tSETTINGS.P2R = tSETTINGS.P2R + 5
        if tSETTINGS.P2R>255 then tSETTINGS.P2R = 0 end
      else end
    elseif x>670 and x<720 and y>(i*40)+40 and y<(i*40)+68 then
      if list[i]=="Player1" then 
        tSETTINGS.P1G = tSETTINGS.P1G + 5
        if tSETTINGS.P1G>255 then tSETTINGS.P1G = 0 end
      elseif list[i]=="Player2" then
        tSETTINGS.P2G = tSETTINGS.P2G + 5
        if tSETTINGS.P2G>255 then tSETTINGS.P2G = 0 end
      else end
    elseif x>730 and x<780 and y>(i*40)+40 and y<(i*40)+68 then
      if list[i]=="Player1" then 
        tSETTINGS.P1B = tSETTINGS.P1B + 5
        if tSETTINGS.P1B>255 then tSETTINGS.P1B = 0 end
      elseif list[i]=="Player2" then
        tSETTINGS.P2B = tSETTINGS.P2B + 5
        if tSETTINGS.P2B>255 then tSETTINGS.P2B = 0 end
      else end
    else end
  end
  if x>300 and x<500 and y>535 and y<575 then
    writesingletable("settings.txt", tSETTINGS)
    startmenu()
  end
end

function clickplayername(x,y)  -- Type in a new name
  local row1 = {"Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P", "DEL"}
  local row2 = {"CAPS", "A", "S", "D", "F", "G", "H", "J", "K", "L", "ENTER"}
  local row3 = {"Z", "X", "C", "V", "B", "N", "M"}
  local i
  if x>25 and x<100 and y>300 and y<340 then gSUBSELECT = 1-gSUBSELECT
  elseif x>700 and x<775 and y>200 and y<240 then
    if gSELECT==1 then 
      if string.len(tSETTINGS.Player1)>=2 then tSETTINGS.Player1 = string.sub(tSETTINGS.Player1, 1, string.len(tSETTINGS.Player1)-1)
      else tSETTINGS.Player1 = "" end
    else
      if string.len(tSETTINGS.Player2)>=2 then tSETTINGS.Player2 = string.sub(tSETTINGS.Player2, 1, string.len(tSETTINGS.Player2)-1)
      else tSETTINGS.Player2 = "" end
    end
  elseif x>675 and x<775 and y>300 and y<340 then
    writesingletable("settings.txt", tSETTINGS)
    gGAMEMODE = "settings"
  elseif y>200 and y<240 then
    for i = 1, #row1, 1 do
      if x>(i*50)+75 and x<(i*50)+125 and row1[i]~="DEL" then
        if gSELECT==1 and string.len(tSETTINGS.Player1)<15 then
          if gSUBSELECT==0 then tSETTINGS.Player1 = tSETTINGS.Player1..string.lower(row1[i])
          else tSETTINGS.Player1 = tSETTINGS.Player1..row1[i] end
        elseif gSELECT==2 and string.len(tSETTINGS.Player2)<15 then
          if gSUBSELECT==0 then tSETTINGS.Player2 = tSETTINGS.Player2..string.lower(row1[i])
          else tSETTINGS.Player2 = tSETTINGS.Player2..row1[i] end
        else end
      end
    end
  elseif y>300 and y<340 then
    for i = 1, #row2, 1 do
      if x>(i*50)+50 and x<(i*50)+100 and row2[i]~="CAPS" and row2[i]~="ENTER" then
        if gSELECT==1 and string.len(tSETTINGS.Player1)<15 then
          if gSUBSELECT==0 then tSETTINGS.Player1 = tSETTINGS.Player1..string.lower(row2[i])
          else tSETTINGS.Player1 = tSETTINGS.Player1..row2[i] end
        elseif gSELECT==2 and string.len(tSETTINGS.Player2)<15 then
          if gSUBSELECT==0 then tSETTINGS.Player2 = tSETTINGS.Player2..string.lower(row2[i])
          else tSETTINGS.Player2 = tSETTINGS.Player2..row2[i] end
        else end
      end
    end
  elseif y>400 and y<440 then
    for i = 1, #row3, 1 do
      if x>(i*50)+125 and x<(i*50)+175 then
        if gSELECT==1 and string.len(tSETTINGS.Player1)<15 then
          if gSUBSELECT==0 then tSETTINGS.Player1 = tSETTINGS.Player1..string.lower(row3[i])
          else tSETTINGS.Player1 = tSETTINGS.Player1..row3[i] end
        elseif gSELECT==2 and string.len(tSETTINGS.Player2)<15 then
          if gSUBSELECT==0 then tSETTINGS.Player2 = tSETTINGS.Player2..string.lower(row3[i])
          else tSETTINGS.Player2 = tSETTINGS.Player2..row3[i] end
        else end
      end
    end
  else end
end

function love.mousereleased(x, y, button, istouch, presses)  -- Record any release, based on gamemode
  x,y = x/gRATIOX, y/gRATIOY
  if _G["release"..gGAMEMODE]~=nil then _G["release"..gGAMEMODE](x,y) end
  gMOUSEGRAB = 0
  gDRAGGING = false
end

function releasegame(x,y)  -- Released mouse main game
  if gDRAGGING==true then
    if x>=xBOARD-(tSETTINGS.Boardsize/2) and x<=xBOARD+(tSETTINGS.Boardsize/2) and y>=yBOARD-(tSETTINGS.Boardsize/2) and y<=yBOARD+(tSETTINGS.Boardsize/2) then releaseboard(x,y)    -- Released mouse on the board
    end
  end
end

function releaseboard(x,y)   -- Released mouse on the board
  local gx, gy = screentogrid(x,y)
  local pl, index = getsquare(gx,gy)
  local i, j
  if gACTION=="Play" and tGRID[gy][gx]==true then
    playcard(tCLICK.Player, tCLICK.Index, gx, gy)
  elseif gACTION=="Move" and tGRID[gy][gx]==true then
    if pl==0 and index==0 then
      movecard(tCLICK.Player, tCLICK.Index, gx, gy)
    elseif pl~=gTURN then
      if gethasability(tCLICK.Card, "Swipe") then       -- Swipe three spaces together
        if tCLICK.Card.GX==tCOMBAT[pl][index].GX then      -- On same column
          if tCOMBAT[pl][index].GX-1>=1 then 
            i, j = getsquare(tCOMBAT[pl][index].GX-1, tCOMBAT[pl][index].GY)
            if i~=0 and i~=gTURN then
              addfloatytext(tCOMBAT[i][j], "-Swipe-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Swipe")
            end
          end
          pl, index = getsquare(gx,gy)
          if tCOMBAT[pl][index].GX+1<=tGAME.Gridsize then 
            i, j = getsquare(tCOMBAT[pl][index].GX+1, tCOMBAT[pl][index].GY)
            if i~=0 and i~=gTURN then 
              addfloatytext(tCOMBAT[i][j], "-Swipe-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Swipe")
            end
          end
          pl, index = getsquare(gx,gy)
        elseif tCLICK.Card.GY==tCOMBAT[pl][index].GY then  -- On same row
          if tCOMBAT[pl][index].GY-1>=1 then 
            i, j = getsquare(tCOMBAT[pl][index].GX, tCOMBAT[pl][index].GY-1)
            if i~=0 and i~=gTURN then
              addfloatytext(tCOMBAT[i][j], "-Swipe-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Swipe")
            end
          end
          pl, index = getsquare(gx,gy)
          if tCOMBAT[pl][index].GY+1<=tGAME.Gridsize then 
            i, j = getsquare(tCOMBAT[pl][index].GX, tCOMBAT[pl][index].GY+1)
            if i~=0 and i~=gTURN then 
              addfloatytext(tCOMBAT[i][j], "-Swipe-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Swipe")
            end
          end
          pl, index = getsquare(gx,gy)
        else end
      end
      if gethasability(tCLICK.Card, "Tail Whip") then    -- Attack two opposite spaces together
        diffx = tCOMBAT[pl][index].GX-tCLICK.Card.GX
        diffy = tCOMBAT[pl][index].GY-tCLICK.Card.GY
        if diffx~=0 or diffy~=0 then
          i, j = tCLICK.Card.GX-diffx, tCLICK.Card.GY-diffy
          if i>=1 and i<=tGAME.Gridsize and j>=1 and j<=tGAME.Gridsize then
            i, j = getsquare(i, j)
            if i~=0 and i~=gTURN then
              addfloatytext(tCOMBAT[i][j], "-Tail Whip-")
              attackcard(tCLICK.Player, tCLICK.Index, i, j, "Tail Whip")
            end
          end
        end
      end
      attackcard(tCLICK.Player, tCLICK.Index, pl, index)
    else end
  else end
end

function gridtoscreen(x,y)  -- Convert grid to screen coords
  local sq_size = tSETTINGS.Boardsize/tGAME.Gridsize
  return xBOARD-(tGAME.Gridsize*sq_size/2)+((x-1)*sq_size)+(sq_size/2), yBOARD-(tGAME.Gridsize*sq_size/2)+((y-1)*sq_size)+(sq_size/2)
end

function screentogrid(x, y)  -- Convert screen to grid coords
  local sq_size = tSETTINGS.Boardsize/tGAME.Gridsize
  return math.floor(((x-xBOARD)/sq_size)+0.5)+((tGAME.Gridsize+1)/2), math.floor(((y-yBOARD)/sq_size)+0.5)+((tGAME.Gridsize+1)/2)
end

function getlistofcards(list, class, tier, sortby)  -- Get a sorted list of cards
  local unsorted, sorted, selected = {}, {}, {}
  local i, j, score, best, ref
  class = class or "All"
  tier = tier or 0
  sortby = sortby or "Tier"
  if next(list)~=nil then
    for i = 1, #list, 1 do
      if (class == "All" or list[i].Class==class) and (tier == 0 or list[i].Tier==tier) then table.insert(unsorted, deepcopy(list[i])) end
    end
  end
  if next(unsorted)~=nil then
    for j = 1, #unsorted, 1 do
      selected[j] = false
    end
    for j = 1, #selected, 1 do
      best, ref = 0, 0
      for i = 1, #unsorted, 1 do
        if selected[i]==false then
          if sortby == "Tier" then 
            score = (11-unsorted[i].Tier)*10
            if score>best or (score==best and (ref==0 or unsorted[i].Name<unsorted[ref].Name)) then
              best = score
              ref = i
            end
          else
            if ref==0 or unsorted[i].Name<unsorted[ref].Name then ref = i end
          end
        end
      end
      if ref ~= 0 then
        table.insert(sorted, deepcopy(unsorted[ref]))
        selected[ref] = true
      end
    end
  end
  return sorted
end

function getisnameinlist(card, deck)  -- Is the card in the deck (by name)
  local i
  local found = false
  if next(deck)~=nil then
    for i = 1, #deck, 1 do
      if card.Name==deck[i].Name then found = true end
    end
  end
  return found
end

function getiscardunlocked(card)   -- Is this card unlocked (by name)
  local i
  local found = false
  if next(tUNLOCKED)~=nil then
    for i = 1, #tUNLOCKED, 1 do
      if card.Name==tUNLOCKED[i] then found = true end
    end
  end
  return found
end

function removecardfromdeck(card, pl)  -- Remove this card from player's deck (by name)
  local i
  if next(tDECK[pl])~=nil then
    for i = #tDECK[pl], 1, -1 do
      if tDECK[pl][i].Name==card.Name then table.remove(tDECK[pl], i) end
    end
  end
end

function addcardtodeck(card, pl)  -- Add this card to the player's deck
  table.insert(tDECK[pl], deepcopy(card))
end

function addcardtocollection(name)  -- Add a card to the player's collection (by name)
  local i
  if next(tBLUEPRINT)~=nil then
    for i = 1, #tBLUEPRINT, 1 do
      if tBLUEPRINT[i].Name==name then 
        table.insert(tCOLLECTION, deepcopy(tBLUEPRINT[i]))
        savedeck(tCOLLECTION, "collection.txt")
      end
    end
  end
end

function getpower(card)  -- Get attack power of card
  local p = card.Power
  if gethaseffect(card, "Nighttime") then p = p + 1 end
  if gethaseffect(card, "Daytime") then p = p - 1 end
  if gethasability(card, "Rage") and card.Wounds~=nil and card.Wounds>0 then p = p + 1 end
  p = math.max(p, 0)
  return math.floor(p+0.5)
end

function gethealth(card)  -- Get health of card
  local h = card.MaxHealth
  if card.Wounds~=nil then h = h - card.Wounds end
  return h
end

function getabilitybyname(name)  -- Get ability number by name of ability
  local a = 0
  local i
  if next(tABILITIES)~=nil then
    for i = 1, #tABILITIES, 1 do
      if tABILITIES[i].Name==name then a = i end
    end
  end
  return a
end

function getpatternbyname(name)  -- Get pattern number by name of movement pattern
  local p = 0
  local i
  if next(tPATTERN)~=nil then
    for i = 1, #tPATTERN, 1 do
      if tPATTERN[i].Name==name then p = i end
    end
  end
  return p
end

function getdimensions(size)  -- Get dimensions (rows, cols) of a rectangle that will encompass this many spaces
  local rows, cols, i = 1, 1, 2
  while rows*cols<size do
    if i==1 then 
      rows = rows + 1
      i = 2
    else 
      cols = cols + 1 
      i = 1
    end
  end
  return rows, cols
end

function getisfreespace(gx, gy)  -- Is this grid space free?
  local i, j
  local free = true
  for j = 1, 2, 1 do
    if next(tCOMBAT[j])~=nil then
      for i = 1, #tCOMBAT[j], 1 do
        if tCOMBAT[j][i].GX==gx and tCOMBAT[j][i].GY==gy then free = false end
      end
    end
  end
  return free
end

function getsquare(gx, gy)    -- Returns player, index
  local pl, index = 0, 0
  local i, j
  for j = 1, 2, 1 do
    if next(tCOMBAT[j])~=nil then
      for i = 1, #tCOMBAT[j], 1 do
        if tCOMBAT[j][i].GX==gx and tCOMBAT[j][i].GY==gy then pl, index = j, i end
      end
    end
  end
  return pl, index
end

function gethasfreeendspace(pl)  -- Does this player have a free zone to play a card into?
  local free = false
  local i
  for i = 1, tGAME.Gridsize, 1 do
    if pl==1 and tGRID[tGAME.Gridsize][i]==true then free = true
    elseif pl==2 and tGRID[1][i]==true then free = true
    else end
  end
  return free
end

function gethasability(card, ability)  -- Does this card have this ability?
  local found = false
  local i
  for i = 1, 3, 1 do
    if card["Spec"..i]~=nil and card["Spec"..i]==ability then found = true end
  end
  return found
end

function gethaseffect(card, effect)  -- Does this card have this effect?
  local id = geteffectbyname(effect)
  local found = false
  if id~=0 and card.Effect~=nil and card.Effect[id]>0 then found = true end
  return found
end

function seteffect(effect, amount, source, pl, index, maxmin)   -- Set a card's effect by name
  local id = geteffectbyname(effect)
  maxmin = maxmin or "Default"
  if id~=0 then
    if source=="Deck" then
      if maxmin=="Default" then tDECK[pl][index].Effect[id]=amount
      elseif maxmin=="Max" then tDECK[pl][index].Effect[id]=math.max(tDECK[pl][index].Effect[id], amount)
      elseif maxmin=="Min" then tDECK[pl][index].Effect[id]=math.min(tDECK[pl][index].Effect[id], amount)
      else end
    elseif source=="Combat" then
      if maxmin=="Default" then tCOMBAT[pl][index].Effect[id]=amount
      elseif maxmin=="Max" then tCOMBAT[pl][index].Effect[id]=math.max(tCOMBAT[pl][index].Effect[id], amount)
      elseif maxmin=="Min" then tCOMBAT[pl][index].Effect[id]=math.min(tCOMBAT[pl][index].Effect[id], amount)
      else end
    else end
  end
end

function geteffectbyname(name)      -- Get effect number by name
  local id = 0
  local i
  if next(tEFFECTS)~=nil then
    for i = 1, #tEFFECTS, 1 do
      if tEFFECTS[i].Name==name then id = i end
    end
  end
  return id
end

function getcolours(pl)   -- Returns player colours
  if pl==1 then return tSETTINGS.P1R, tSETTINGS.P1G, tSETTINGS.P1B
  else return tSETTINGS.P2R, tSETTINGS.P2G, tSETTINGS.P2B end
end

function readcollection(list)  -- Read the player's collection of cards
  local i, j, found
  tCLASS, tTIER = {}, {}
  if next(list)~=nil then
    for i = 1, #list, 1 do
      found = false
      if next(tCLASS)~=nil then
        for j = 1, #tCLASS, 1 do
          if list[i].Class == tCLASS[j] then found = true end
        end
      end
      if found == false then table.insert(tCLASS, list[i].Class) end
      found = false
      if next(tTIER)~=nil then
        for j = 1, #tTIER, 1 do
          if list[i].Tier == tTIER[j] then found = true end
        end
      end
      if found == false then table.insert(tTIER, list[i].Tier) end
    end
  end
  if next(tCLASS)~=nil and #tCLASS>1 then
    for i = 1, #tCLASS, 1 do
      for j = 1, #tCLASS-1, 1 do
        if tCLASS[j]=="Other" or (tCLASS[j] > tCLASS[j+1] and tCLASS[j+1]~="Other") then tCLASS[j], tCLASS[j+1] = tCLASS[j+1], tCLASS[j] end
      end
    end
  end
end

function highlightgrid()  -- Highlight squares that are available
  local gx, gy, i, j, pl, index, blocked
  -- Blank the grid first
  for j = 1, tGAME.Gridsize, 1 do
    for i = 1, tGAME.Gridsize, 1 do
      tGRID[j][i]=false
    end
  end
  -- Now highlight according to actions
  if gACTION == "Play" then
    for i = 1, tGAME.Gridsize, 1 do
      if tCLICK.Player==1 then
        pl, index = getsquare(i, tGAME.Gridsize)
        if pl==0 and index==0 then tGRID[tGAME.Gridsize][i]=true end
      elseif tCLICK.Player==2 then
        pl, index = getsquare(i, 1)
        if pl==0 and index==0 then tGRID[1][i]=true end
      else end
    end
  elseif gACTION == "Move" then
    gx, gy = tCLICK.Card.GX, tCLICK.Card.GY
    if tCLICK.Card.Pattern=="Alpha" then 
      for i = -1, 1, 2 do
        if gx+i>=1 and gx+i<=tGAME.Gridsize then  -- 1 space horizontal
          pl, index = getsquare(gx+i, gy)
          if pl~=tCLICK.Player then tGRID[gy][gx+i]=true end
        end  
        if gy+i>=1 and gy+i<=tGAME.Gridsize then  -- 1 space vertical
          pl, index = getsquare(gx, gy+i)
          if pl~=tCLICK.Player then tGRID[gy+i][gx]=true end
        end
      end
    elseif tCLICK.Card.Pattern=="Beta" then 
      for j = -1, 1, 1 do
        for i = -1, 1, 1 do
          if (i~=0 or j~=0) and gx+i>=1 and gx+i<=tGAME.Gridsize and gy+j>=1 and gy+j<=tGAME.Gridsize then  -- 1 space horizonal, vertical or diagonal
            pl, index = getsquare(gx+i, gy+j)
            if pl~=tCLICK.Player then tGRID[gy+j][gx+i]=true end
          end  
        end
      end
    elseif tCLICK.Card.Pattern=="Gamma" then 
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, 2, 1 do
          if gx+(j*i)>=1 and gx+(j*i)<=tGAME.Gridsize and blocked==false then  -- Horizontal
            pl, index = getsquare(gx+(j*i), gy)
            if pl~=tCLICK.Player then tGRID[gy][gx+(j*i)]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, 2, 1 do
          if gy+(j*i)>=1 and gy+(j*i)<=tGAME.Gridsize and blocked==false then  -- Vertical
            pl, index = getsquare(gx, gy+(j*i))
            if pl~=tCLICK.Player then tGRID[gy+(j*i)][gx]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
    elseif tCLICK.Card.Pattern=="Delta" then 
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, 2, 1 do
          if gx+(j*i)>=1 and gx+(j*i)<=tGAME.Gridsize and blocked==false then  -- Horizontal
            pl, index = getsquare(gx+(j*i), gy)
            if pl~=tCLICK.Player then tGRID[gy][gx+(j*i)]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, 2, 1 do
          if gy+(j*i)>=1 and gy+(j*i)<=tGAME.Gridsize and blocked==false then  -- Vertical
            pl, index = getsquare(gx, gy+(j*i))
            if pl~=tCLICK.Player then tGRID[gy+(j*i)][gx]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, 2, 1 do
          if gx+(j*i)>=1 and gx+(j*i)<=tGAME.Gridsize and gy+(j*i)>=1 and gy+(j*i)<=tGAME.Gridsize and blocked==false then  -- Diagonal 1
            pl, index = getsquare(gx+(j*i), gy+(j*i))
            if pl~=tCLICK.Player then tGRID[gy+(j*i)][gx+(j*i)]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, 2, 1 do
          if gx+(j*i)>=1 and gx+(j*i)<=tGAME.Gridsize and gy-(j*i)>=1 and gy-(j*i)<=tGAME.Gridsize and blocked==false then  -- Diagonal 1
            pl, index = getsquare(gx+(j*i), gy-(j*i))
            if pl~=tCLICK.Player then tGRID[gy-(j*i)][gx+(j*i)]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
    elseif tCLICK.Card.Pattern=="Epsilon" then 
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, tGAME.Gridsize-1, 1 do
          if gx+(j*i)>=1 and gx+(j*i)<=tGAME.Gridsize and blocked==false then  -- Horizontal
            pl, index = getsquare(gx+(j*i), gy)
            if pl~=tCLICK.Player then tGRID[gy][gx+(j*i)]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, tGAME.Gridsize-1, 1 do
          if gy+(j*i)>=1 and gy+(j*i)<=tGAME.Gridsize and blocked==false then  -- Vertical
            pl, index = getsquare(gx, gy+(j*i))
            if pl~=tCLICK.Player then tGRID[gy+(j*i)][gx]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
    elseif tCLICK.Card.Pattern=="Zeta" then 
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, tGAME.Gridsize-1, 1 do
          if gx+(j*i)>=1 and gx+(j*i)<=tGAME.Gridsize and blocked==false then  -- Horizontal
            pl, index = getsquare(gx+(j*i), gy)
            if pl~=tCLICK.Player then tGRID[gy][gx+(j*i)]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, tGAME.Gridsize-1, 1 do
          if gy+(j*i)>=1 and gy+(j*i)<=tGAME.Gridsize and blocked==false then  -- Vertical
            pl, index = getsquare(gx, gy+(j*i))
            if pl~=tCLICK.Player then tGRID[gy+(j*i)][gx]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, tGAME.Gridsize-1, 1 do
          if gx+(j*i)>=1 and gx+(j*i)<=tGAME.Gridsize and gy+(j*i)>=1 and gy+(j*i)<=tGAME.Gridsize and blocked==false then  -- Diagonal 1
            pl, index = getsquare(gx+(j*i), gy+(j*i))
            if pl~=tCLICK.Player then tGRID[gy+(j*i)][gx+(j*i)]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
      for j = -1, 1, 2 do
        blocked = false
        for i = 1, tGAME.Gridsize-1, 1 do
          if gx+(j*i)>=1 and gx+(j*i)<=tGAME.Gridsize and gy-(j*i)>=1 and gy-(j*i)<=tGAME.Gridsize and blocked==false then  -- Diagonal 1
            pl, index = getsquare(gx+(j*i), gy-(j*i))
            if pl~=tCLICK.Player then tGRID[gy-(j*i)][gx+(j*i)]=true end
            if pl~=0 and index~=0 and not gethasability(tCLICK.Card, "Fly") then blocked=true end   -- There was a piece here
          end
        end
      end
    elseif tCLICK.Card.Pattern=="Eta" then 
      for j = 1, tGAME.Gridsize, 1 do
        for i = 1, tGAME.Gridsize, 1 do
          if ((math.abs(i-gx)==1 and math.abs(j-gy)==2) or (math.abs(i-gx)==2 and math.abs(j-gy)==1)) then  -- Jumping knight move
            pl, index = getsquare(i, j)
            if pl~=tCLICK.Player then tGRID[j][i]=true end
          end  
        end
      end
    else end
    -- Now unhighlight squares that have an opponent in if I can't attack
    if gethaseffect(tCLICK.Card, "Passive") then
      for j = 1, tGAME.Gridsize, 1 do
        for i = 1, tGAME.Gridsize, 1 do
          if tGRID[j][i]==true then
            pl, index = getsquare(i, j)
            if pl~=0 or index~=0 then tGRID[j][i]=false end
          end
        end
      end
    end
    -- Unhighlight charm mammal
    if tCLICK.Card.Class=="Mammal" then
      for j = 1, tGAME.Gridsize, 1 do
        for i = 1, tGAME.Gridsize, 1 do
          pl, index = getsquare(i, j)
          if pl==3-tCLICK.Player and gethasability(tCOMBAT[pl][index], "Charm Mammal") then tGRID[j][i]=false end
        end
      end
    end
  else end
end

function playcard(pl, index, i, j)  -- Play card to combat
  local a
  for a = 1, #tDECK[pl], 1 do
    -- Other cards are now unavailable until next turn
    if a~=index and (not gethasability(tDECK[pl][index], "Leader") or not gethasability(tDECK[pl][a], "Follower")) then tDECK[pl][a].Available = false end 
  end
  if not gethasability(tDECK[pl][index], "Surprise Attack") then seteffect("Immobile", 1, "Deck", pl, index, "Max") end
  tDECK[pl][index].Available = false
  tDECK[pl][index].GX, tDECK[pl][index].GY = i, j
  tDECK[pl][index].OriginalX, tDECK[pl][index].OriginalY = tDECK[pl][index].X, tDECK[pl][index].Y
  tDECK[pl][index].TargetX, tDECK[pl][index].TargetY = gridtoscreen(i,j)
  tDECK[pl][index].Animate = true
  tDECK[pl][index].AniCount = tSETTINGS.AniTime
  if gethasability(tDECK[pl][index], "Nocturnal") then -- This creature is nocturnal, set an effect
    if gDAYNIGHT>=6 then 
      addfloatytext(tDECK[pl][index], "+Nocturnal+")
      seteffect("Nighttime", 1, "Deck", pl, index, "Max")
    else 
      addfloatytext(tDECK[pl][index], "-Nocturnal-")
      seteffect("Daytime", 1, "Deck", pl, index, "Max")
    end
  end
  if gethasability(tDECK[pl][index], "Stealthy") then -- This creature is stealthy, set an effect
    addfloatytext(tDECK[pl][index], "+Stealthy+")
    seteffect("Hidden", 100, "Deck", pl, index, "Max")
  end
  if gethasability(tDECK[pl][index], "Defend the Queen") then   -- Defend the Queen ability, is the queen present?
    if next(tCOMBAT[pl])~=nil then
      for a = 1, #tCOMBAT[pl], 1 do
        if tCOMBAT[pl][a].Name=="Queen Ant" and string.find(tDECK[pl][index].Name, "Ant") then
          tDECK[pl][index].MaxHealth = tDECK[pl][index].MaxHealth + 1
          tDECK[pl][index].Power = tDECK[pl][index].Power + 1
          addfloatytext(tDECK[pl][index], "+Defend the Queen+")
        end
      end
    end
  elseif tDECK[pl][index].Name=="Queen Ant" then   -- I am the Queen, is anyone my loyal subject?
    if next(tCOMBAT[pl])~=nil then
      for a = 1, #tCOMBAT[pl], 1 do
        if gethasability(tCOMBAT[pl][a], "Defend the Quee") and string.find(tCOMBAT[pl][a].Name, "Ant") then
          tCOMBAT[pl][a].MaxHealth = tCOMBAT[pl][a].MaxHealth + 1
          tCOMBAT[pl][a].Power = tCOMBAT[pl][a].Power + 1
          addfloatytext(tCOMBAT[pl][a], "+Defend the Queen+")
        end
      end
    end
  else end
  table.insert(tCOMBAT[pl], deepcopy(tDECK[pl][index]))
  table.remove(tDECK[pl], index)
  arrangehand(pl)
  clearclick()
  highlightgrid()
end

function movecard(pl, index, i, j)  -- Move card to square
  local energetic, industrious, confused = false, false, false
  local cx, cy = {}, {}
  local x, y, a, b
  for x = 1, #tCOMBAT[pl], 1 do     -- All other combat cards now immobile
    if x~=index and (not gethasability(tCOMBAT[pl][index], "Leader") or not gethasability(tCOMBAT[pl][x], "Follower")) then seteffect("Immobile", 1, "Combat", pl, x, "Max") end
  end
  if gethasability(tCOMBAT[pl][index], "Energetic") and math.random(100)<=25 then energetic = true
  elseif gethasability(tCOMBAT[pl][index], "Industrious") and math.random(100)<=33 then industrious = true
  else end
  if gethasability(tCOMBAT[pl][index], "Confused") and math.random(100)<=25 and AIsimulation==false then confused = true end
  if confused == true then
    for y = 1, tGAME.Gridsize, 1 do
      for x = 1, tGAME.Gridsize, 1 do
        a, b = getsquare(x, y)
        if tGRID[y][x]==true and a==0 and b==0 then
          table.insert(cx, x)
          table.insert(cy, y)
        end
      end
    end
    if next(cx)~=nil and next(cy)~=nil and #cx>1 and #cy>1 then
      repeat
        a = math.random(#cx)
        x, y = cx[a], cy[a]
      until cx[a]~=i or cy[a]~=j
      i, j = cx[a], cy[a]
      addfloatytext(tCOMBAT[pl][index], "-Confused-")
    end
  end
  tCOMBAT[pl][index].GX, tCOMBAT[pl][index].GY = i, j
  tCOMBAT[pl][index].OriginalX, tCOMBAT[pl][index].OriginalY = tCOMBAT[pl][index].X, tCOMBAT[pl][index].Y
  tCOMBAT[pl][index].TargetX, tCOMBAT[pl][index].TargetY = gridtoscreen(i, j)
  tCOMBAT[pl][index].Animate = true
  tCOMBAT[pl][index].AniCount = tSETTINGS.AniTime
  if energetic==true then addfloatytext(tCOMBAT[pl][index], "+Energetic+")
  elseif industrious==true then addfloatytext(tCOMBAT[pl][index], "+Industrious+")
  else seteffect("Immobile", 1, "Combat", pl, index, "Max") end
  calculatezones()
  clearclick()
  highlightgrid()
end

function attackcard(att_pl, att_ind, def_pl, def_ind, special)  -- Attack opponent card
  local breeds, web, bees, spiky = false, false, false, false
  local x, y = {}, {}
  local i, j, pl, index, card
  special = special or "None"  -- Some special attacks don't move the attacking creature e.g. Swipe, Tail Whip
  for i = 1, #tCOMBAT[att_pl], 1 do     -- All other combat cards are now immobile
    if i~=att_ind then seteffect("Immobile", 1, "Combat", att_pl, i, "Max") end
  end
  if gethasability(tCOMBAT[att_pl][att_ind], "Breeds") and math.random(100)<=33 then breeds = true end
  if gethasability(tCOMBAT[def_pl][def_ind], "Web") and tCOMBAT[att_pl][att_ind].Class=="Insect" and (tCOMBAT[att_pl][att_ind].Size=="Small" or tCOMBAT[att_pl][att_ind].Size=="Tiny") then web = true end
  if gethasability(tCOMBAT[def_pl][def_ind], "Swarm of Bees") then bees = true end
  if gethasability(tCOMBAT[def_pl][def_ind], "Spiky") then
    addfloatytext(tCOMBAT[def_pl][def_ind], "-Spiky-")
    spiky = true 
  end
  -- Neither creature is now hidden
  seteffect("Hidden", 0, "Combat", att_pl, att_ind, "Min")
  seteffect("Hidden", 0, "Combat", def_pl, def_ind, "Min")
  if gethasability(tCOMBAT[att_pl][att_ind], "Rage") and tCOMBAT[att_pl][att_ind].Wounds>0 then addfloatytext(tCOMBAT[att_pl][att_ind], "+Rage+") end
  -- Attack animation
  tCOMBAT[att_pl][att_ind].OriginalX, tCOMBAT[att_pl][att_ind].OriginalY = tCOMBAT[att_pl][att_ind].X, tCOMBAT[att_pl][att_ind].Y
  tCOMBAT[att_pl][att_ind].TargetX, tCOMBAT[att_pl][att_ind].TargetY = tCOMBAT[def_pl][def_ind].X, tCOMBAT[def_pl][def_ind].Y
  tCOMBAT[att_pl][att_ind].Animate = true
  tCOMBAT[att_pl][att_ind].AniCount = tSETTINGS.AniTime
  if gethasability(tCOMBAT[def_pl][def_ind], "Evasive") and math.random(100)<=25 then   -- Did the defender evade the attack?
    addfloatytext(tCOMBAT[def_pl][def_ind], "+Evaded+")
    addfloatytext(tCOMBAT[att_pl][att_ind], "-Missed-")
  elseif gethasability(tCOMBAT[def_pl][def_ind], "Imitate Call") and math.random(100)<=50 and (tCOMBAT[att_pl][att_ind].Class=="Bird" or tCOMBAT[att_pl][att_ind].Class=="Mammal") then   -- Or did they imitate the call and dodge them?
    addfloatytext(tCOMBAT[def_pl][def_ind], "+Imitate+")
    addfloatytext(tCOMBAT[att_pl][att_ind], "-Missed-")
  elseif gethasability(tCOMBAT[def_pl][def_ind], "Trickery") and math.random(100)<=50 then    -- Or did they trick the attacker?
    addfloatytext(tCOMBAT[def_pl][def_ind], "+Trickery+")
    addfloatytext(tCOMBAT[att_pl][att_ind], "-Missed-")
  else  -- Otherwise attack them!
    if gethasability(tCOMBAT[att_pl][att_ind], "Paralysis") then    -- Paralysis
      seteffect("Immobile", 2, "Combat", def_pl, def_ind, "Max")
      addfloatytext(tCOMBAT[def_pl][def_ind], "-Paralysed-")
    end
    if gethasability(tCOMBAT[att_pl][att_ind], "Poison") then      -- Poison
      seteffect("Venom", 100, "Combat", def_pl, def_ind, "Max")
      addfloatytext(tCOMBAT[def_pl][def_ind], "-Poisoned-")
    end
    if gethasability(tCOMBAT[att_pl][att_ind], "Snake Coil") and math.random(100)<=75 then    -- Snake Coil
      seteffect("Limp", 2, "Combat", def_pl, def_ind, "Max")
      addfloatytext(tCOMBAT[def_pl][def_ind], "-Limp-")
    end
    damage(tCOMBAT[def_pl][def_ind], getpower(tCOMBAT[att_pl][att_ind]))
    if gethasability(tCOMBAT[att_pl][att_ind], "Blood Sucking") then
      tCOMBAT[att_pl][att_ind].MaxHealth = tCOMBAT[att_pl][att_ind].MaxHealth + 1
      addfloatytext(tCOMBAT[att_pl][att_ind], "+Blood Sucking+")
    end
    if gethasability(tCOMBAT[att_pl][att_ind], "Headbutt") then  -- Headbutt, knocks defender one 'backwards'
      i = sign(tCOMBAT[def_pl][def_ind].GX-tCOMBAT[att_pl][att_ind].GX)
      j = sign(tCOMBAT[def_pl][def_ind].GY-tCOMBAT[att_pl][att_ind].GY)
      i, j = tCOMBAT[def_pl][def_ind].GX+i, tCOMBAAT[def_pl][def_ind].GY+j
      if i>=1 and i<=tGAME.Gridsize and j>=1 and j<=tGAME.Gridsize then
        pl, index = getsquare(i, j)
        if pl==0 and index==0 then
          tCOMBAT[def_pl][def_ind].GX, tCOMBAT[def_pl][def_ind].GY = i, j
          tCOMBAT[def_pl][def_ind].OriginalX, tCOMBAT[def_pl][def_ind].OriginalY = tCOMBAT[def_pl][def_ind].X, tCOMBAT[def_pl][def_ind].Y
          tCOMBAT[def_pl][def_ind].Animate = true
          tCOMBAT[def_pl][def_ind].AniCount = tSETTINGS.AniTime
          tCOMBAT[def_pl][def_ind].TargetX, tCOMBAT[def_pl][def_ind].TargetY = gridtoscreen(i, j)
        end
      end
    end
  end
  if gethealth(tCOMBAT[def_pl][def_ind])<=0 then  -- Is the defender dead?
    if special~="Swipe" and special~="Tail Whip" then 
      tCOMBAT[att_pl][att_ind].GX, tCOMBAT[att_pl][att_ind].GY = tCOMBAT[def_pl][def_ind].GX, tCOMBAT[def_pl][def_ind].GY
      if gethasability(tCOMBAT[att_pl][att_ind], "Ferocious") then
        tCOMBAT[att_pl][att_ind].Power = tCOMBAT[att_pl][att_ind].Power + 1
        addfloatytext(tCOMBAT[att_pl][att_ind], "+Increased attack+")
      end
    end
    if gethasability(tCOMBAT[def_pl][def_ind], "Regrow") and math.random(100)<=50 then  -- Regrow, return to hand
      tCOMBAT[def_pl][def_ind].GX, tCOMBAT[def_pl][def_ind].GY = 0, 0
      tCOMBAT[def_pl][def_ind].Wounds = 0
      for i = 1, #tEFFECTS, 1 do
        tCOMBAT[def_pl][def_ind].Effect[i] = 0
      end
      table.insert(tDECK[def_pl], deepcopy(tCOMBAT[def_pl][def_ind]))
      arrangehand(def_pl)
      tDECK[def_pl][#tDECK[def_pl]].TargetX, tDECK[def_pl][#tDECK[def_pl]].TargetY = tDECK[def_pl][#tDECK[def_pl]].X, tDECK[def_pl][#tDECK[def_pl]].Y
      tDECK[def_pl][#tDECK[def_pl]].X, tDECK[def_pl][#tDECK[def_pl]].Y = tDECK[def_pl][#tDECK[def_pl]].OriginalX, tDECK[def_pl][#tDECK[def_pl]].OriginalY
    else
      -- Go to the grave
      table.insert(tGRAVE[def_pl], deepcopy(tCOMBAT[def_pl][def_ind]))
    end
    table.remove(tCOMBAT[def_pl], def_ind)
  end
  if special~="Swipe" and special~="Tail Whip" then
    gACTION = ""
    gZOOMED = false
    tCLICK = {}
    highlightgrid()
    if web==true then seteffect("Immobile", 3, "Combat", att_pl, att_ind, "Max") end
    if gethasability(tCOMBAT[att_pl][att_ind], "Energetic") and math.random(100)<=25 then addfloatytext(tCOMBAT[att_pl][att_ind], "+Energetic+")
    elseif gethasability(tCOMBAT[att_pl][att_ind], "Industrious") and math.random(100)<=33 then addfloatytext(tCOMBAT[att_pl][att_ind], "+Industrious+")
    else seteffect("Immobile", 1, "Combat", att_pl, att_ind, "Max") end
  end
  if breeds == true then
    -- Find a free adjacent space
    for j = tCOMBAT[att_pl][att_ind].GY-1, tCOMBAT[att_pl][att_ind].GY+1, 1 do
      for i = tCOMBAT[att_pl][att_ind].GX-1, tCOMBAT[att_pl][att_ind].GX+1, 1 do
        if j>=1 and j<=tGAME.Gridsize and i>=1 and i<=tGAME.Gridsize then
          pl, index = getsquare(i, j)
          if pl==0 and index==0 then 
            table.insert(x, i)
            table.insert(y, j)
          end
        end
      end
    end
    if next(x)~=nil and next(y)~=nil then
      i = math.random(#x)
      addfloatytext(tCOMBAT[att_pl][att_ind], "+Breeds+")
      table.insert(tCOMBAT[att_pl], deepcopy(tCOMBAT[att_pl][att_ind]))
      tCOMBAT[att_pl][#tCOMBAT[att_pl]].Wounds = 0
      tCOMBAT[att_pl][#tCOMBAT[att_pl]].GX = x[i]
      tCOMBAT[att_pl][#tCOMBAT[att_pl]].GY = y[i]
    end
  end
  calculatezones()
  if gethasability(tCOMBAT[att_pl][att_ind], "Single Use") then   -- Single Use attacker, go to grave
    addfloatytext(tCOMBAT[att_pl][att_ind], "-Single Use-")
    tCOMBAT[att_pl][att_ind].OriginalX, tCOMBAT[att_pl][att_ind].OriginalY = tCOMBAT[att_pl][att_ind].X, tCOMBAT[att_pl][att_ind].Y
    tCOMBAT[att_pl][att_ind].Animate = true
    tCOMBAT[att_pl][att_ind].AniCount = tSETTINGS.AniTime
    tCOMBAT[att_pl][att_ind].TargetX, tCOMBAT[att_pl][att_ind].TargetY = -100, 300
    table.insert(tGRAVE[att_pl], deepcopy(tCOMBAT[att_pl][att_ind]))
    table.remove(tCOMBAT[att_pl], att_ind)
  elseif spiky == true then    -- Spiky, take one damage
    damage(tCOMBAT[att_pl][att_ind], 1)
    if gethealth(tCOMBAT[att_pl][att_ind])<=0 then
      table.insert(tGRAVE[att_pl], deepcopy(tCOMBAT[att_pl][att_ind]))
      table.remove(tCOMBAT[att_pl], att_ind)
    end
  else end
  if bees == true then     -- Swarm of Bees
    if next(tCOMBAT[att_pl])~=nil then
      for i = #tCOMBAT[att_pl], 1, -1 do
        damage(tCOMBAT[att_pl][i], 1, true)
        if gethealth(tCOMBAT[att_pl][i])<=0 then
          table.insert(tGRAVE[att_pl], deepcopy(tCOMBAT[att_pl][i]))
          table.remove(tCOMBAT[att_pl], i)
        end
      end
    end
  end
end

function damage(card, amount, ignore)  -- damage card by amount, ignore armour?
  local dead = false
  local i, pl, index
  ignore = ignore or false
  if ignore==false then
    for i = 1, 3, 1 do
      if gethasability(card, "Armour +"..i) then
        addfloatytext(card, "+Armoured+")
        amount = amount - i
      end
    end
  end
  if amount > 0 then
    card.Wounds = card.Wounds + amount
    addfloatytext(card, "-"..amount.." damage-")
    if gethealth(card)<=0 then
      dead = true
      addfloatytext(card, "-Death-")
      card.OriginalX, card.OriginalY = card.X, card.Y
      card.TargetX, card.TargetY = -100, 300
      card.Animate = true
      card.AniCount = tSETTINGS.AniTime
      --Check for carrion ability
      if card.GX>1 then
        pl, index = getsquare(card.GX-1, card.GY)
        if pl~=0 and gethasability(tCOMBAT[pl][index], "Carrion") then
          tCOMBAT[pl][index].MaxHealth = tCOMBAT[pl][index].MaxHealth + 1
          addfloatytext(tCOMBAT[pl][index], "+Carrion+")
        end
      end
      if card.GX<tGAME.Gridsize then
        pl, index = getsquare(card.GX+1, card.GY)
        if pl~=0 and gethasability(tCOMBAT[pl][index], "Carrion") then
          tCOMBAT[pl][index].MaxHealth = tCOMBAT[pl][index].MaxHealth + 1
          addfloatytext(tCOMBAT[pl][index], "+Carrion+")
        end
      end
      if card.GY>1 then
        pl, index = getsquare(card.GX, card.GY-1)
        if pl~=0 and gethasability(tCOMBAT[pl][index], "Carrion") then
          tCOMBAT[pl][index].MaxHealth = tCOMBAT[pl][index].MaxHealth + 1
          addfloatytext(tCOMBAT[pl][index], "+Carrion+")
        end
      end
      if card.GY<tGAME.Gridsize then
        pl, index = getsquare(card.GX, card.GY+1)
        if pl~=0 and gethasability(tCOMBAT[pl][index], "Carrion") then
          tCOMBAT[pl][index].MaxHealth = tCOMBAT[pl][index].MaxHealth + 1
          addfloatytext(tCOMBAT[pl][index], "+Carrion+")
        end
      end
    end
  end
  return dead
end

function playsound(i, interrupt)  -- Play sound
  interrupt = interrupt or "No"
  if sSOUND[i]~=nil and tSETTINGS.Sound=="Yes" then 
    if interrupt == "Yes" then 
      sSOUND[i]:stop()
      love.audio.play(sSOUND[i])
    else
      if not sSOUND[i]:isPlaying() then love.audio.play(sSOUND[i]) end
    end
  end
end

function addfloatytext(card, text)  -- Add floatytext to card
  entry = {}
  entry.Text = text
  entry.X, entry.Y = card.X, card.Y
  entry.Timer = 0
  table.insert(card.FloatyText, entry)
end

function animatetext(card)  -- Animate floatytext
  local i = 0
  repeat
    i = i + 1
    card.FloatyText[i].Timer = card.FloatyText[i].Timer + 1
  until i==#card.FloatyText or card.FloatyText[i].Timer <= 40
  if card.FloatyText[1].Timer >= tSETTINGS.FloatTime then table.remove(card.FloatyText, 1) end
end

function animatecard(card)  -- Animate card
  card.X = card.X + ((card.TargetX - card.OriginalX) / tSETTINGS.AniTime)
  card.Y = card.Y + ((card.TargetY - card.OriginalY) / tSETTINGS.AniTime)
  card.AniCount = card.AniCount - 1
  if card.AniCount <= 0 then
    card.AniCount = 0
    if gethealth(card)>0 and card.GX~=0 and card.GY~=0 then card.X, card.Y = gridtoscreen(card.GX, card.GY)
    else card.X, card.Y = card.TargetX, card.TargetY end
    card.Animate = false
  end
end

function scoregrid(pl)  -- Score the grid as it stands
  local ALLY_BASE, ALLY_HEALTH, ALLY_ATTACK = 0.5, 5, 5
  local OPP_BASE, OPP_HEALTH, OPP_ATTACK = 0.5, 5, 5
  local PROTECTIVE, AGGRESSION = 1, 0.8
  local MIDDLE_FACTOR, CONTROL_FACTOR = 50, 20
  local ZONE_MIN, ZONE_MAX = 100, 1000
  local RANDOM = 10
  local SCORE_COMBAT, SCORE_DANGER, SCORE_CONTROL, SCORE_ZONE, SCORE_RANDOM = 0, 0, 0, 0, 0
  local gridcontrol = {}
  local middle = (tGAME.Gridsize+1)/2
  local i, j, x, y

  -- Blank gridcontrol
  for j = 1, tGAME.Gridsize, 1 do
    gridcontrol[j] = {}
    for i = 1, tGAME.Gridsize, 1 do
      gridcontrol[j][i] = {}
      gridcontrol[j][i].MaxDefense = 0
      gridcontrol[j][i].MaxThreat = 0
      gridcontrol[j][i].TotDefense = 0
      gridcontrol[j][i].TotThreat = 0
    end
  end

  -- Blank the highlight grid
  clearclick()
  highlightgrid()

  -- Loop through all combat, score combat according to number/health/attack of allies and also adjust gridcontrol
  for j = 1, 2, 1 do
    if next(tCOMBAT[j])~=nil then
      for i = 1, #tCOMBAT[j], 1 do
        -- Score points for allies on the grid taking into account their cost, health and attack
        if j==pl then SCORE_COMBAT=SCORE_COMBAT+(tCOMBAT[j][i].Cost*ALLY_BASE)+(getpower(tCOMBAT[j][i])*ALLY_ATTACK)+(gethealth(tCOMBAT[j][i])*ALLY_HEALTH)
        else SCORE_COMBAT=SCORE_COMBAT-(tCOMBAT[j][i].Cost*OPP_BASE)-(getpower(tCOMBAT[j][i])*OPP_ATTACK)-(gethealth(tCOMBAT[j][i])*OPP_HEALTH) end
        -- Adjust gridcontrol based upon this creature's possible moves
        gACTION = "Move"
        clickcard(tCOMBAT[j][i], "Combat", j, i)
        highlightgrid()
        for y = 1, tGAME.Gridsize, 1 do
          for x = 1, tGAME.Gridsize, 1 do
            if tGRID[y][x]==true then
              if j==pl then 
                gridcontrol[y][x].MaxDefense = math.max(gridcontrol[y][x].MaxDefense, getpower(tCOMBAT[j][i]))
                gridcontrol[y][x].TotDefense = gridcontrol[y][x].TotDefense + getpower(tCOMBAT[j][i])
              else 
                gridcontrol[y][x].MaxThreat = math.max(gridcontrol[y][x].MaxThreat, getpower(tCOMBAT[j][i]))
                gridcontrol[y][x].TotThreat = gridcontrol[y][x].TotThreat + getpower(tCOMBAT[j][i])
              end
            end
          end
        end
        clearclick()
        highlightgrid()
      end
    end
  end

  -- Loop through all combat again now we know gridcontrol values
  for j = 1, 2, 1 do
    if next(tCOMBAT[j])~=nil then
      for i = 1, #tCOMBAT[j], 1 do
        -- Is the creature somewhere it could die? Score this positively/negatively multiplied by protective/aggression
        if j==pl and gridcontrol[tCOMBAT[j][i].GY][tCOMBAT[j][i].GX].MaxThreat>=gethealth(tCOMBAT[j][i]) then SCORE_DANGER=SCORE_DANGER-(((tCOMBAT[j][i].Cost*ALLY_BASE)-(getpower(tCOMBAT[j][i])*ALLY_ATTACK)-(gethealth(tCOMBAT[j][i])*ALLY_HEALTH))*PROTECTIVE)
        elseif j==3-pl and gridcontrol[tCOMBAT[j][i].GY][tCOMBAT[j][i].GX].MaxDefense>=gethealth(tCOMBAT[j][i]) then SCORE_DANGER=SCORE_DANGER+(((tCOMBAT[j][i].Cost*OPP_BASE)+(getpower(tCOMBAT[j][i])*OPP_ATTACK)+(gethealth(tCOMBAT[j][i])*OPP_HEALTH))*AGGRESSION)
        else end
        -- Having my allies closer to the middle is a good thing
        y = math.abs(tCOMBAT[j][i].GY-middle)
        y = (middle-y)/middle
        if j==pl then SCORE_CONTROL=SCORE_CONTROL+(y*MIDDLE_FACTOR)
        else SCORE_CONTROL=SCORE_CONTROL-(y*MIDDLE_FACTOR) end
      end
    end
  end

  -- Score gridcontrol, closest to the middle gets more points
  for y = 1, tGAME.Gridsize, 1 do
    for x = 1, tGAME.Gridsize, 1 do
      i = math.abs(y-middle)
      i = (middle-i)/middle
      SCORE_CONTROL = SCORE_CONTROL + (sign(gridcontrol[y][x].TotDefense-gridcontrol[y][x].TotThreat))*(i*CONTROL_FACTOR)
    end
  end

  -- Having control of the middle zone is a very good thing, which gets better still approaching the end of the game
  for x = 1, tGAME.Gridsize, 1 do
    if tZONE[x]==pl then SCORE_ZONE = SCORE_ZONE + ZONE_MIN + ((gROUND/tGAME.Rounds)*(ZONE_MAX-ZONE_MIN))
    elseif tZONE[x]==3-pl then SCORE_ZONE = SCORE_ZONE - ZONE_MIN - ((gROUND/tGAME.Rounds)*(ZONE_MAX-ZONE_MIN))
    else end
  end

  -- Random factor
  SCORE_RANDOM = math.random(RANDOM) - math.random(RANDOM)

  -- Round the values and return
  SCORE_COMBAT = math.floor(SCORE_COMBAT+0.5)
  SCORE_DANGER = math.floor(SCORE_DANGER+0.5)
  SCORE_CONTROL = math.floor(SCORE_CONTROL+0.5)
  SCORE_ZONE = math.floor(SCORE_ZONE+0.5)
  --print("Combat: "..SCORE_COMBAT)
  --print("Danger: "..SCORE_DANGER)
  --print("Control: "..SCORE_CONTROL)
  --print("Zones: "..SCORE_ZONE)
  --print("Random: "..SCORE_RANDOM)
  return SCORE_COMBAT+SCORE_DANGER+SCORE_CONTROL+SCORE_ZONE+SCORE_RANDOM
end

function AI()  -- Run the AI for its turn
  local backuptCLICK = deepcopy(tCLICK)   -- backup of what has been clicked
  local backupZOOM = gZOOMED              -- backup of zoom state
  local backupGRAB = gGRABBED             -- backup of grab state
  local cardstoplay, spacestoplay = {}, {}  -- list of cards I can play and where I can play them
  local xmove, ymove = {}, {}             -- list of spaces I can move to (or attack)
  local aiDECK, aiCOMBAT, aiGRAVE, aiTURN, aiWAIT
  local i, j, pl, index, score, entry, card
  if gTURN==1 then entry=tGAME.Gridsize
  else entry=1 end
  clearclick()
  if AIloop < tSETTINGS.AIMaxLoops then   -- I'm still searching for moves...
    AIsimulation = true                   -- This isn't for real then
    aiDECK = deepcopy(tDECK)              -- save the state of play
    aiCOMBAT = deepcopy(tCOMBAT)          -- save the state of play
    aiGRAVE = deepcopy(tGRAVE)            -- save the state of play
    aiZONE = deepcopy(tZONE)              -- save the state of play
    aiTURN = gTURN                        -- save the state of play
    aiWAIT = gWAIT                        -- save the state of play
    -- What cards can the AI play from hand?
    if next(tDECK[gTURN])~=nil then
      for i = 1, #tDECK[gTURN], 1 do
        if tDECK[gTURN][i].Available==true then table.insert(cardstoplay, i) end
      end
    end
    -- Are there any spaces available for them to go in?
    for i = 1, tGAME.Gridsize, 1 do
      pl, index = getsquare(i, entry)
      if pl==0 then table.insert(spacestoplay, i) end
    end
    if next(cardstoplay)~=nil and next(spacestoplay)~=nil then   -- If I can play a card into battle, always my best first move!
      i = cardstoplay[math.random(#cardstoplay)]
      j = spacestoplay[math.random(#spacestoplay)]
      clickcard(tDECK[gTURN][i], "Deck", gTURN, i)
      gACTION = "Play"
      highlightgrid()
      playcard(gTURN, i, j, entry)
      score = scoregrid(gTURN)
      if score>AIbest_score then
        AIbest_move = {Move="Play", Source="Deck", Player=gTURN, Index=i, X=j, Y=entry}
        AIbest_score = score
      end
    elseif next(tCOMBAT[gTURN]) then  -- Otherwise move a card if I have one to play
      card = math.random(#tCOMBAT[gTURN])
      if not gethaseffect(tCOMBAT[gTURN][card], "Immobile") and not gethaseffect(tCOMBAT[gTURN][card], "Limp") then
        clickcard(tCOMBAT[gTURN][card], "Combat", gTURN, i)
        gACTION = "Move"
        highlightgrid()
        -- Where can I play the card?
        for j = 1, tGAME.Gridsize, 1 do
          for i = 1, tGAME.Gridsize, 1 do
            if tGRID[j][i]==true then
              table.insert(xmove, i)
              table.insert(ymove, j)
            end
          end
        end
        if next(xmove)~=nil and next(ymove)~=nil then  -- Is there a legitimate space?
          i = math.random(#xmove)
          j, i = ymove[i], xmove[i]
          pl, index = getsquare(i, j)
          if pl==0 and index==0 then movecard(gTURN, card, i, j)
          else attackcard(gTURN, card, pl, index) end
          score = scoregrid(gTURN)
          if score>AIbest_score then
            if pl==0 and index==0 then AIbest_move = {Move="Move", Source="Combat", Player=gTURN, Index=card, X=i, Y=j}
            else AIbest_move = {Move="Attack", Source="Combat", Player=gTURN, Index=card, DefPlayer=pl, DefIndex=index} end
            AIbest_score = score
          end
        end
      end
    else end
    tDECK = deepcopy(aiDECK)              -- restore the state of play
    tCOMBAT = deepcopy(aiCOMBAT)          -- restore the state of play
    tGRAVE = deepcopy(aiGRAVE)            -- restore the state of play
    tZONE = deepcopy(aiZONE)              -- restore the state of play
    gTURN = aiTURN                        -- restore the state of play
    gWAIT = aiWAIT                        -- restore the state of play
    AIloop = AIloop + 1                   -- Increase the counter towards making a move
  else                                    -- Its time to move!
    AIsimulation = false                  -- We're not playing around anymore!
    if next(AIbest_move)~=nil then        -- I've got a move worth doing...
      if AIbest_move.Move == "Play" then
        clickcard(tDECK[AIbest_move.Player][AIbest_move.Index], "Deck", AIbest_move.Player, AIbest_move.Index)
        gACTION = "Play"
        highlightgrid()
        playcard(AIbest_move.Player, AIbest_move.Index, AIbest_move.X, AIbest_move.Y)
      elseif AIbest_move.Move == "Move" then
        clickcard(tCOMBAT[AIbest_move.Player][AIbest_move.Index], "Combat", AIbest_move.Player, AIbest_move.Index)
        gACTION = "Move"
        highlightgrid()
        movecard(AIbest_move.Player, AIbest_move.Index, AIbest_move.X, AIbest_move.Y)
      elseif AIbest_move.Move == "Attack" then
        clickcard(tCOMBAT[AIbest_move.Player][AIbest_move.Index], "Combat", AIbest_move.Player, AIbest_move.Index)
        gACTION = "Move"
        highlightgrid()
        attackcard(AIbest_move.Player, AIbest_move.Index, AIbest_move.DefPlayer, AIbest_move.DefIndex)
      else end
      score = scoregrid(gTURN)
      AIbest_move = {}
      AIbest_score = score
      AIloop = 0
    else                                  -- There's no good move for me, skip a turn
      if gROUND==tGAME.Rounds then processresult()
      else newturn() end
    end
  end
  tCLICK = deepcopy(backuptCLICK)
  gZOOMED = backupZOOM
  gGRABBED = backupGRAB
end

function clearclick()  -- Clear the previous click
  tCLICK = {}
  gACTION = ""
  gZOOMED = false
  gGRABBED = false
end